#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Essential PyFRAP module containing :py:class:`embryo` class. 
"""

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Numpy/Scipy
import numpy as np

#PyFRAP Modules
from pyfrp.modules import pyfrp_misc_module 
from pyfrp.modules import pyfrp_img_module 
from pyfrp.modules import pyfrp_fit_module 
from pyfrp.modules import pyfrp_stats_module 
from pyfrp.modules import pyfrp_IO_module
from pyfrp.modules import pyfrp_plot_module
from pyfrp.modules.pyfrp_term_module import *

#PyFRAP Objects
import pyfrp_geometry
import pyfrp_simulation
import pyfrp_analysis
import pyfrp_ROI
import pyfrp_fit

#matplotlib
import matplotlib.pyplot as plt

#Time 
import time

#Misc
import os


#===========================================================================================================================================================================
#Class definitions
#===========================================================================================================================================================================


class embryo:
	
	"""Main PyFRAP class, gathering all the data and parameters of FRAP experiment.
	
	The ``embryo`` class basically stores:
	
		* A minimum set of basic FRAP parameters.
		* A list of :py:class:`pyfrp.subclasses.pyfrp_ROI.ROI` classes, describing all ROIs used for 
		  for evaluating simulation and analysis results.
		* A :py:class:`pyfrp.subclasses.pyfrp_geometry.geometry` class describing the 3-dimensional 
		  geometry of the experiment.
		* A :py:class:`pyfrp.subclasses.pyfrp_analysis.analysis` class describing how the dataset is 
		  going to be analyzed.
		* A :py:class:`pyfrp.subclasses.pyfrp_simulation.simulation` class describing how the dataset is 
		  going to be simulated.
		* A list of :py:class:`pyfrp.subclasses.pyfrp_fit.fit` classes, storing different fitting options
		  and results.
	
	The ``embryo`` class comes with a comprehensive set of methods aimed at making it as powerful as possible,
	while still keeping it simple. The hierarchical structure should make it easy to navigate through a FRAP
	dataset.
	
	"""
	
	#Creates new embryo object
	def __init__(self,name):
		
		#Dataset name
		self.name = name
		
		#Data Image specifics
		self.dataEnc='uint16'
		self.dataFT='tif'
		self.dataResPx=512
		self.dataResMu=322.34
		
		#DataSet files
		self.fileList=[]
		self.fnDatafolder=""
		
		#Data time specifics
		self.frameInterval=10
		self.nFrames=300
		self.tStart=0
		self.tEnd=self.tStart+self.frameInterval*(self.nFrames-1)
		self.tvecData=np.linspace(self.tStart,self.tEnd,self.nFrames)
		
		#Imaging specifics (In mu)
		self.sliceDepthMu=30.
		self.sliceWidthMu=2.5
		
		#Zoom Imaging (need to call this different)
		self.convFact=self.computeConvFact(updateDim=False)
		
		#Imaging specifics (In pixels)
		self.sliceWidthPx=self.sliceWidthMu/self.convFact
		self.sliceDepthPx=self.sliceDepthMu/self.convFact
		self.sliceHeightPx=-self.sliceDepthPx
		self.sliceBottom=0
		
		#Bleaching specifics (in Mu)
		self.sideLengthBleachedMu=2*79.54
		
		#Bleaching specifics (in pxs) (can take out)
		self.sideLengthBleachedPx=self.sideLengthBleachedMu/self.convFact
		self.offsetBleachedPx=[self.dataResPx/2-self.sideLengthBleachedPx/2, self.dataResPx/2-self.sideLengthBleachedPx/2]
		
		#List of ROIs
		self.ROIs=[]
		
		#Master ROI
		self.masterROIIdx=None
		
		#Geometry
		self.geometry=None
		
		#Simulation
		self.simulation=None
		
		#Analysis
		self.analysis=None
		
		#Fitting 
		self.fits=[]

	def addFit(self,fit):
		
		"""Appends fit object to list of fits
		
		Args:
			fit (pyfrp.subclasses.pyfrp_fit.fit): fit object.
			
		Returns:
			list: Updated ``fits`` list.
		
		"""
		
		self.fits.append(fit)
		return self.fits
	
	def newFit(self,name):
		
		"""Creates new fit object and appends it to list of fits.
		
		Args:
			name (str): Name of fit object.
			
		Returns:
			pyfrp.subclasses.pyfrp_fit.fit: Newly created fit.
		
		"""
		
		fit=pyfrp_fit.fit(self,name)
		self.addFit(fit)
		return fit
		
	def deleteFit(self,i):
		
		"""Deletes fit with index ``i`` from ``fits`` list.
		
		Args:
			i (int): Index of fit to be deleted.
			
		Returns:
			list: Updated ``fits`` list.
		
		"""
		
		self.fits.pop(i)
		return self.fits
	
		
	def save(self,fn=None,copyMeshFiles=True,debug=False):
		
		"""Saves embryo object to pickle file.
		
		If ``fn=None`` will save to ``self.name.emb``.
		
		Keyword Args:
			fn (str): Output filename.
			copyMeshFiles (bool): Copy meshfiles to embryo file destination.
			debug (bool): Print out debugging messages.
			
		Returns:
			str: Output filename.
		
		"""
		
		if fn==None:
			fn=self.name+".emb"
		
		if copyMeshFiles:
			
			print "before copyMeshFiles:", fn
			print "fnGeo", self.geometry.fnGeo
			
			fnGeo,fnMesh=pyfrp_IO_module.copyMeshFiles(fn,self.geometry.fnGeo,self.simulation.mesh.fnMesh,debug=debug)
			self.geometry.setFnGeo(fnGeo)
			self.simulation.mesh.setFnMesh(fnMesh)
			
			fnNew=os.path.splitext((os.path.split(fn)[-1]))[0]
			self.renameMeshFiles(fn=fnNew,debug=debug)
		
		pyfrp_IO_module.saveToPickle(self,fn=fn)
		
		print "Saved "+  self.name+ " to " + fn
		return fn
	
	def renameMeshFiles(self,fn=None,debug=False):
		
		"""Renames meshfiles associated with embryo fo ``fn``.
		
		If ``fn=None`` will rename to ``self.name.ext``, where ``ext`` is 
		either ``.geo`` or ``.msh``.
		
		Example: 
		
		>>> emb.geometry.getFnGeo()
		>>> path/to/meshfiles/dome.geo
		>>> emb.getName()
		>>> myEmbryo
		>>> emb.renameMeshFiles()
		>>> emb.geometry.getFnGeo()
		>>> path/to/meshfiles/myEmbryo.geo
		
		.. note:: Will automatically update ``geometry.fnGeo`` and 
		   ``simulation.mesh.fnMesh`` properties.
		
		Keyword Args:
			fn (str): Desired filename.
			debug (bool): Print out debugging messages.
		
		Returns:
			tuple: Tuple containing:
				
				* fnGeoNew (str): New path to geometry file.
				* fnMeshNew (str): New path to mesh file.
				
		"""
		
		if fn==None:
			fn=self.name+".emb"
		
		if self.geometry!=None:
			fnGeoNew=pyfrp_IO_module.copyAndRenameFile(self.geometry.fnGeo,fn,debug=debug)
			self.geometry.setFnGeo(fnGeoNew)
				
		if self.simulation.mesh.mesh!=None:
			fnMeshNew=pyfrp_IO_module.copyAndRenameFile(self.simulation.mesh.fnMesh,fn,debug=debug)
			self.simulation.mesh.setFnMesh(fnMeshNew)
			
		return self.geometry.fnGeo, self.simulation.mesh.fnMesh	
		
	
	def copy(self):
		
		"""Copies embryo, preserving all attributes and methods.
		
		Returns:
			pyfrp.subclasses.pyfrp_embryo.embryo: Embryo copy.
		
		"""
		
		copiedEmbryo=cpy.deepcopy(self)
		return copiedEmbryo
	
	def updateVersion(self):
		
		"""Updates embryo object to current version, making sure that it possesses
		all attributes.
		
		Creates a new embryo object and compares ``self`` with the new embryo object.
		If the new embryo object has a attribute that ``self`` does not have, will
		add attribute with default value from the new embryo object.
		
		
		Returns:
			pyfrp.subclasses.pyfrp_embryo.embryo: ``self``
			
		"""
		
		embtemp=embryo("temp")
		pyfrp_misc_module.updateObj(embtemp,self)
		return self
	
	def computeConvFact(self,updateDim=True):
		
		"""Computes conversion factor between um to px (unit=um/px).
		
		If ``updateDimensions`` is selected, will update all dimensions of
		embryo object data rely on ``convFact``.
		
		Keyword Args:
			updateDim (bool): Automatically updated all convFact relevant attributes.
		
		Returns:
			float: New conversion factor.
			
		"""
		
		self.convFact = self.dataResMu/self.dataResPx
		if updateDim:
			self.updatePxDimensions()
		return self.convFact
	
	def updatePxDimensions(self):
		
		"""Updates all all convFact relevant attributes.
		
		Returns:
			tuple: Tuple containing:
			
				* sliceWidthPx (float): Updated slice width in px.
				* sliceDepthPx (float): Updated slice depth in px.
				* sliceHeightPx (float): Updated slice height in px.
				* sideLengthBleachedPx (float): Updated side length of bleached square in px.
				* offsetBleachedPx (float): Updated offset if bleached square in px.
				
		"""
		
		
		self.sliceWidthPx=self.sliceWidthMu/self.convFact
		self.sliceDepthPx=self.sliceDepthMu/self.convFact
		self.sliceHeightPx=-self.sliceDepthPx
			
		self.updateBleachedRegion()

		return self.sliceWidthPx,self.sliceDepthPx, self.sliceHeightPx, self.sideLengthBleachedPx, self.offsetBleachedPx
	
	def setName(self,n):
		
		"""Sets embryo name."""
		
		self.name=n
		return self.name
	
	def getName(self):
		
		"""Returns embryo name."""
		
		return self.name
	
	def setDataResMu(self,res):
		
		"""Sets resolution of data in :math:`\mu m`."""
		
		self.dataResMu=res 
		self.computeConvFact()
		return self.dataResMu
	
	def setDataResPx(self,res):
		
		"""Sets resolution of data in px. """
		
		self.dataResPx=int(res) 
		self.computeConvFact()
		return self.dataResPx
	
	def setDataFT(self,f):
		
		"""Sets data filetype of datasets, for example *.tif* ."""
		
		self.dataFT=f 
		return self.dataFT
	
	def setDataEnc(self,e):
		
		"""Sets data encoding of datasets, for example ``uint16`` ."""
		
		self.dataEnc=e 
		return self.dataEnc
	
	def getDataFT(self):
		
		"""Returns current data filetype."""
		
		return self.dataFT
	
	def getDataEnc(self):
		
		"""Returns current data encoding."""
		
		return self.dataEnc
	
	def setFrameInterval(self,dt):
		
		"""Sets interval between imaging frames, then updates all time vectors.
		
		Args: 
			dt (float): New frame interval in seconds.
		
		Returns: 
			float: Set frame interval.
		
		"""
		
		self.frameInterval=dt
		self.updateTimeDimensions()
		return self.frameInterval
	
	def setTStart(self,t):
		
		"""Sets start time of experiment, then updates all time vectors.
		
		Args: 
			t (float): Start time in seconds.
		
		Returns: 
			float: Set start time.
		
		"""
		
		self.tStart=t
		self.updateTimeDimensions()
		return self.tStart
	
	def setTEnd(self,t):
		
		"""Sets end time of experiment, then updates all time vectors.
		
		Args: 
			t (float): End time in seconds.
		
		Returns: 
			float: Set end time.
		
		"""
		
		self.tEnd=t
		self.updateTimeDimensions()
		return self.tEnd
	
	def updateNFrames(self):
		
		"""Updates number of frames, then updates all time vectors.
		
		.. note:: Gets number of frames by reading number of files of type ``dataFT`` in ``fnDatafolder``. 
		   So you should make sure that there are no extra files in ``fnDatafolder``.
		
		
		Returns: 
			int: Updated number of frames.
		
		"""
		
		self.nFrames=len(self.fileList)
		self.updateTimeDimensions()
		return self.nFrames
	
	def getNFrames(self):
		
		"""Returns current number of frames."""
		
		return self.nFrames
	
	def setNFrames(self,n):
		
		"""Sets number of frames, then updates all time vectors. """
		
		self.nFrames=n
		self.updateTimeDimensions()
		return self.nFrames
	
	def getDataResPx(self):
		
		"""Returns resolution of data in px."""
		
		return self.dataResPx
	
	def getDataResMu(self):
		
		"""Returns resolution of data in :math:`\um m` ."""
		
		return self.dataResMu
	
	def getFrameInterval(self):
		
		"""Returns current frame interval."""
		
		return self.frameInterval
	
	def getTStart(self):
		
		"""Returns current exerpiment start time."""
		
		return self.tStart
	
	def getTEnd(self):
		
		"""Returns current exerpiment end time."""
		
		return self.tEnd
	
	def getTvecData(self):
		
		"""Returns current data time vector."""
		
		return self.tvecData
	
	def updateTimeDimensions(self,simUpdate=True):
		
		"""Updates time dimensions using information in ``frameInterval``, ``nFrames`` 
		and ``tStart``.
		
		.. note:: If embryo object already possesses simulation object, will update simulation
		   time dimensions using :py:func:`pyfrp.subclasses.pyfrp_simulation.simulation.toDefaultTvec` .
		   If you have different settings in simulation vector that you want to preserve, select
		   ``simUpdate=False``.
		   
		Keyword Args:
			simUpdate (bool): Update simulation time dimensions.
		
		Returns:
			numpy.ndarray: Updated data time vector.
		
		"""
		
		self.tEnd=self.tStart+self.frameInterval*(self.nFrames-1)
		self.tvecData=np.linspace(self.tStart,self.tEnd,self.nFrames)
		if self.simulation!=None:
			if simUpdate:
				self.simulation.toDefaultTvec()
		return self.tvecData
	
	def setSliceDepthMu(self,d,updateGeometry=True):
		
		"""Sets resolution of data in :math:`\mu m` and then updates all dimensions of
		embryo object data rely on ``sliceDepthMu``.
		
		If ``updateGeometry`` is selected, will automatically update geometry.
		
		.. note:: Not every geometry depends on slice depth. If the geometry object has
		   a method ``restoreDefault``, this will be called. Otherwise geometry is not updated.
		
		Args:
			d (float): New slice depth in :math:`\mu m`.
			
		Keyword Args:
			updateGeometry (bool): Update geometry.
			
		Returns:
			float: New slice depth.
			
		"""
		
		self.sliceDepthMu=d
		self.updatePxDimensions()
		
		if updateGeometry:
			if hasattr(self.geometry,"restoreDefault"):
				self.geometry.restoreDefault()
			
		
		return self.sliceDepthMu
	
	def setMasterROIIdx(self,idx):
		
		"""Define ROI with index ``idx`` as master ROI.
		"""
		
		self.masterROIIdx=idx
		return self.masterROIIdx
	
	def setSideLengthBleachedMu(self,s):
		
		"""Sets sidelength of bleached square in :math:`\mu m`, will then 
		update bleached region parameters by calling :py:func:`updateBleachedRegion`.
		
		.. warning:: Attributes ``sideLengthBleachedMu`` and ``offsetBleachedPx`` will 
		   deprecated in further versions. Bleached region definition will then solely rely 
		   on ROI definitions. 
		
		"""
		
		self.sideLengthBleachedMu=s
		self.updateBleachedRegion()
		return self.sideLengthBleachedMu
		
	def updateBleachedRegion(self):
		
		r"""Updates sidelength and offset of bleached square in px.
		
		Bleached region is defined around center of image. That is, the offset is set to
		   
		.. math:: x_{\mathrm{offset}} = \frac{1}{2}(r_{\mathrm{px}},r_{\mathrm{px}})^T - (s_{\mathrm{bleached,px}},s_{\mathrm{bleached,px}})^T  
		   
		where :math:`r_{\mathrm{px}}` is the resolution of the data in pixel, and :math:`s_{\mathrm{bleached,px}` is
		the sidelength of the bleached square in pixel.
		
		.. warning:: Attributes ``sideLengthBleachedMu`` and ``offsetBleachedPx`` will 
		   deprecated in further versions. Bleached region definition will then solely rely 
		   on ROI definitions. 
		
		"""
		
		self.sideLengthBleachedPx=self.sideLengthBleachedMu/self.convFact
		self.offsetBleachedPx=[self.dataResPx/2-self.sideLengthBleachedPx/2, self.dataResPx/2-self.sideLengthBleachedPx/2]
		return self.sideLengthBleachedPx, self.offsetBleachedPx
		
	def getMasterROIIdx(self):
		
		"""Returns index of master ROI"""
		
		return self.masterROIIdx
	
	def getROIIdx(self,r):
		
		"""Returns index of ROI in ``ROIs`` list.
		
		Returns ``-1`` if ROI is not in list.
		
		Args:
			r (pyfrp.subclasses.pyfrp_ROI.ROI): Some ROI.
			
		Returns:
			int: Index of ROI.
			
		"""
		
		try:
			return self.ROIs.index(r)
		except ValueError:
			printError("ROI "+ r.name+ " is not in ROIs list of embryo "+self.name+".")
			return -1
	
	def addROI(self,roi):
		
		"""Adds ROI to ``ROIs`` list.
		
		Args:
			roi (pyfrp.subclasses.pyfrp_ROI.ROI): A PyFRAP ROI.
			
		Returns:
			list: Updated ROIs list.
		
		"""
	
		self.ROIs.append(roi)
		
		return self.ROIs
	
	def newROI(self,name,Id,zmin='-inf',zmax='inf',color='b',asMaster=False):
		
		"""Creates new simple :py:class:`pyfrp.subclasses.pyfrp_ROI.ROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			
		Keyword Args:
			zmin (float): Lower boundary in z-direction.
			zmax (float): upper boundary in z-direction.
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.ROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.ROI(self,name,Id,zmin=zmin.inf,zmax=zmax,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
		
	def newRadialROI(self,name,Id,center,radius,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.radialROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			center (list): Center of radial ROI.
			radius (float): Radius of radial ROI.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.radialROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.radialROI(self,name,Id,center,radius,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
		
	def newSliceROI(self,name,Id,height,width,sliceBottom,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.sliceROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		.. note:: If ``sliceBottom==True``, slice ranges from ``height<=z<=height+width``, otherwise
		   from ``height-width/2<=z<=height+width/2``.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			height (float): z-coordinate of slice.
			width (float): width in z-direction of slice.
			sliceBottom (bool): Put origin of slice at botton of slice.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.sliceROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.sliceROI(self,name,Id,height,width,sliceBottom,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newRadialSliceROI(self,name,Id,center,radius,height,width,sliceBottom,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		.. note:: If ``sliceBottom==True``, slice ranges from ``height<=z<=height+width``, otherwise
		   from ``height-width/2<=z<=height+width/2``.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			center (list): Center of radial ROI.
			radius (float): Radius of radial ROI.
			height (float): z-coordinate of slice.
			width (float): width in z-direction of slice.
			sliceBottom (bool): Put origin of slice at botton of slice.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.radialSliceROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.radialSliceROI(self,name,Id,center,radius,height,width,sliceBottom,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newSquareROI(self,name,Id,offset,sidelength,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.squareROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: Offset is set to be left-bottom corner of square.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			offset (list): Offset of of square.
			sidelength (float): Sidelength of square.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.squareROI: Newly created ROI object.
		
		"""
		
		
		roi=pyfrp_ROI.squareROI(self,name,Id,offset,sidelength)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newSquareSliceROI(self,name,Id,offset,sidelength,height,width,sliceBottom,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.squareSliceROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: Offset is set to be left-bottom corner of square.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		.. note:: If ``sliceBottom==True``, slice ranges from ``height<=z<=height+width``, otherwise
		   from ``height-width/2<=z<=height+width/2``.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			center (list): Center of radial ROI.
			offset (list): Offset of of square.
			sidelength (float): Sidelength of square.
			width (float): width in z-direction of slice.
			sliceBottom (bool): Put origin of slice at botton of slice.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.squareSliceROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.squareSliceROI(self,name,Id,offset,sidelength,height,width,sliceBottom,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newRectangleROI(self,name,Id,offset,sidelengthX,sidelengthY,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.rectangleROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: Offset is set to be left-bottom corner of rectangle.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			offset (list): Offset of of square.
			sidelengthX (float): Sidelength of rectangle in x-direction.
			sidelengthY (float): Sidelength of rectangle in y-direction.
				
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.rectangleROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.rectangleROI(self,name,Id,offset,sidelengthX,sidelengthY,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newRectangleSliceROI(self,name,Id,offset,sidelengthX,sidelengthY,height,width,sliceBottom,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.rectangleSliceROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: Offset is set to be left-bottom corner of rectangle.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		.. note:: If ``sliceBottom==True``, slice ranges from ``height<=z<=height+width``, otherwise
		   from ``height-width/2<=z<=height+width/2``.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			center (list): Center of radial ROI.
			offset (list): Offset of of rectangle.
			sidelengthX (float): Sidelength of rectangle in x-direction.
			sidelengthY (float): Sidelength of rectangle in y-direction.
			width (float): width in z-direction of slice.
			sliceBottom (bool): Put origin of slice at botton of slice.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.rectangleSliceROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.rectangleSliceROI(self,name,Id,offset,sidelengthX,sidelengthY,height,width,sliceBottom,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newPolyROI(self,name,Id,corners,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.polyROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		Each corner given in ``corners`` list must be a list of form ``[x,y]``.
		
		.. note:: Polygon is automatically closed.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			corners (list): List of ``[x,y]`` coordinates describing corners.
				
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.polyROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.polyROI(self,name,Id,corners,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newPolySliceROI(self,name,Id,corners,height,width,sliceBottom,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.polySliceROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		Each corner given in ``corners`` list must be a list of form ``[x,y]``.
		
		.. note:: Polygon is automatically closed.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		.. note:: If ``sliceBottom==True``, slice ranges from ``height<=z<=height+width``, otherwise
		   from ``height-width/2<=z<=height+width/2``.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			center (list): Center of radial ROI.
			corners (list): List of ``[x,y]`` coordinates describing corners.
			width (float): width in z-direction of slice.
			sliceBottom (bool): Put origin of slice at botton of slice.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.polySliceROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.polySliceROI(self,name,Id,corners,height,width,sliceBottom,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def newCustomROI(self,name,Id,color='b',asMaster=False):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_ROI.customROI` object
		and adds it to the ``ROIs`` list of embryo object.
		
		.. note:: You can use :py:meth:`getFreeROIId` to find a unused ID for the newROI.
		
		Args:
			name (str): Name of new ROI.
			Id (int): ID of new ROI.
			
		Keyword Args:
			color (str): Color of ROI.
			asMaster (bool): Set ROI as master ROI?
			
		Returns:
			pyfrp.subclasses.pyfrp_ROI.customROI: Newly created ROI object.
		
		"""
		
		roi=pyfrp_ROI.customROI(self,name,Id,color=color)
		self.ROIs.append(roi)
		if asMaster:
			self.masterROIIdx=self.ROIs.index(roi)
		return roi
	
	def getFreeROIId(self):
		
		"""Returns first free ID of ROIs.
		"""
		
		IdList=pyfrp_misc_module.objAttrToList(self.ROIs,'Id')
		notFound=True
		i=0
		while notFound:
			if i not in IdList:
				return i
			i=i+1
			
	def removeROI(self,i):
		
		"""Removes ROI of index ``i``.
		"""
		
		self.ROIs.pop(i)
		if i==self.masterROIIdx:
			printWarning('You are deleting the masterROI. You need to select a new one before you can analyze or simulate.')
		return self.ROIs
	
	def listROIs(self):
		
		"""Prints out all ROIs and their respective type."""
		
		for r in self.ROIs:
			print r.name , type(r)
		return True
	
	def genDefaultROIs(self,center,radius,rimFactor=0.66,masterROI=None,bleachedROI=None,rimROI=None,sliceHeightPx=None,clean=True):
		
		"""Creates a standard set of ROI objects and adds them to ``ROIs`` list.
		
		The set of ROIs covers the generally most useful ROIs used for FRAP experiments, providing already all the settings
		that PyFRAP uses for rim computation etc.
		
		ROIs contain:
		
			* All: ROI covering all pixels and mesh nodes. :py:class:`pyfrp.subclasses.pyfrp_ROI.sliceROI`
			* All Square: ROI covering all pixels and mesh nodes inside bleached region. :py:class:`pyfrp.subclasses.pyfrp_ROI.squareROI`
			* All Out: ROI covering all pixels and mesh nodes outside bleached region. :py:class:`pyfrp.subclasses.pyfrp_ROI.customROI`
			* Slice: ROI covering all pixels and mesh nodes inside recorded field of view. :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
			* Slice rim: ROI covering all pixels that are not used for rim concentration computation. :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
			* Rim: ROI covering all pixels that are used for rim concentration computation. :py:class:`pyfrp.subclasses.pyfrp_ROI.customROI`
			* Bleached Square: ROI covering all pixels and mesh nodes inside bleached region and imaging slice. :py:class:`pyfrp.subclasses.pyfrp_ROI.squareSliceROI`
			* Out: ROI covering all pixels and mesh nodes outside bleached region but inside imaging slice. :py:class:`pyfrp.subclasses.pyfrp_ROI.customROI`
			
		.. note:: Will automatically set ``Slice`` as ``masterROI``.
		
		.. note:: If ``masterROI`` is given, will use this ROI instead of *Slice* as master ROI via :py:func:`setMasterROIIdx`. Will not create *Slice* at all. 
		   If ``masterROI`` is not in ``ROIs`` list yet, it will be automatically added to the list. 
		
		.. note:: If ``bleachedROI`` is given, will use this ROI instead of *Bleached Square*. Will generate copy of ``bleachedROI`` to generate *All Square* type 
		   ROI.
		
		.. note:: If ``rimROI`` is given, *Slice rim* is not created, and ``rimROI`` is used instead. 
		
		.. note:: If ``sliceHeightPx`` is given, will create ROIs *Slice*, *Out*, *Bleached Square* and *Rim* at this height, otherwise will use value stored in ``embryo.sliceHeightPx``.
		
		Args:	
			center (list): Center of circle defining imaging slice.
			radius (float): Radius of circle defining imaging slice.
			
		Keyword Args:	
			rimFactor (float): Factor describing percentage of imaging slice excluded from rim computation.
			sliceHeightPx (float): Height of slice ROI in px.
			clean (bool): Will remove all ROIs from embryo object before creating new ones.
			masterROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI that is supposed to be used as a masterROI.
			bleachedROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI that is supposed to be used to indicate the bleached region.
			rimROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI that is substracted from *Slice*. Should lie withing *Slice*.
			
		Returns:
			list: Updated list of ROIs.
		
		
		"""
		
		#Clean up if necessary
		if clean:
			self.ROIs=[]
			
		#Add masterROI and bleachedROI to ROIs list if they are given (necessary such that getFreeROIId works)
		if masterROI!=None and masterROI not in self.ROIs:
			self.ROIs.append(masterROI)
		if bleachedROI!=None and bleachedROI not in self.ROIs:
			self.ROIs.append(bleachedROI)
		if rimROI!=None and rimROI not in self.ROIs:
			self.ROIs.append(rimROI)
		
		#Check if sliceHeight is given
		if sliceHeightPx==None:
			sliceHeightPx=self.sliceHeightPx
		
		#Create All ROI
		allsl=self.newSliceROI("All",self.getFreeROIId(),0,np.inf,False,color=(0.5,0.3,0.4))
		
		#Create All Square
		if bleachedROI!=None:
			zmin,zmax=allsl.getZExtend()
			allsqu=bleachedROI.getCopy() 
			allsqu.setName("All "+bleachedROI.getName())
			allsqu.setZExtend(zmin,zmax)
			allsqu.setId(self.getFreeROIId())
			allsqu.setColor((1.,140/255.,0.))
			self.ROIs.append(allsqu)
		else:	
			allsqu=self.newSquareROI('All Square',self.getFreeROIId(),self.offsetBleachedPx,self.sideLengthBleachedPx,color=(1.,140/255.,0.))
		
		#Create All Out
		allout=self.newCustomROI("All Out",self.getFreeROIId(),color=(0.3,0.4,0.5))
		allout.addROI(allsl,1)
		allout.addROI(allsqu,-1)
		
		#Create Slice
		if masterROI!=None:
			sl=masterROI
			self.setMasterROIIdx(self.getROIIdx(sl))
			if sl.getName()!="Slice":
				printWarning("masterROI is not named Slice, this might lead to problems.")
				
		else:
			sl=self.newRadialSliceROI("Slice",self.getFreeROIId(),center,radius,sliceHeightPx,self.sliceWidthPx,self.sliceBottom,color='g',asMaster=True)
		
		#Create Rim
		if rimROI!=None:
			sl2=rimROI
		else:
			sl2=self.newRadialSliceROI("Slice rim",self.getFreeROIId(),center,rimFactor*radius,sliceHeightPx,self.sliceWidthPx,self.sliceBottom,color='y')
		 
		rim=self.newCustomROI("Rim",self.getFreeROIId(),color='m')
		rim.addROI(sl,1)
		rim.addROI(sl2,-1)
		rim.setUseForRim(True)
		
		#Create Bleached Square
		if bleachedROI!=None:
			squ=bleachedROI
			if squ.getName()!="Bleached Square":
				printWarning("Bleached ROI is not named Bleached Square, this might lead to problems.")
		else:	
			squ=self.newSquareSliceROI("Bleached Square",self.getFreeROIId(),self.offsetBleachedPx,self.sideLengthBleachedPx,sliceHeightPx,self.sliceWidthPx,self.sliceBottom,color='b')
		
		#Create Out
		out=self.newCustomROI("Out",self.getFreeROIId(),color='r')
		out.addROI(sl,1)
		out.addROI(squ,-1)
		
		return self.ROIs
	
	def genMinimalROIs(self,center,radius,rimFactor=0.66,masterROI=None,bleachedROI=None,rimROI=None,sliceHeightPx=None,clean=True):
		
		"""Creates a minimal standard set of ROI objects and adds them to ``ROIs`` list.
		
		The set of ROIs covers the generally most useful ROIs used for FRAP experiments, providing already all the settings
		that PyFRAP uses for rim computation etc.
		
		ROIs contain:
		
			* Slice: ROI covering all pixels and mesh nodes inside recorded field of view. :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
			* Slice rim: ROI covering all pixels that are not used for rim concentration computation. :py:class:`pyfrp.subclasses.pyfrp_ROI.radialSliceROI`
			* Rim: ROI covering all pixels that are used for rim concentration computation. :py:class:`pyfrp.subclasses.pyfrp_ROI.customROI`
			* Bleached Square: ROI covering all pixels and mesh nodes inside bleached region and imaging slice. :py:class:`pyfrp.subclasses.pyfrp_ROI.squareSliceROI`
			
		.. note:: Will automatically set ``Slice`` as ``masterROI``.
		
		.. note:: If ``masterROI`` is given, will use this ROI instead of *Slice* as master ROI via :py:func:`setMasterROIIdx`. Will not create *Slice* at all. 
		   If ``masterROI`` is not in ``ROIs`` list yet, it will be automatically added to the list. 
		
		.. note:: If ``bleachedROI`` is given, will use this ROI instead of *Bleached Square*. Will generate copy of ``bleachedROI`` to generate *All Square* type 
		   ROI.
		
		.. note:: If ``rimROI`` is given, *Slice rim* is not created, and ``rimROI`` is used instead. 
		
		.. note:: If ``sliceHeightPx`` is given, will create ROIs *Slice*, *Out*, *Bleached Square* and *Rim* at this height, otherwise will use value stored in ``embryo.sliceHeightPx``.
		
		Args:	
			center (list): Center of circle defining imaging slice.
			radius (float): Radius of circle defining imaging slice.
			
		Keyword Args:	
			rimFactor (float): Factor describing percentage of imaging slice excluded from rim computation.
			sliceHeightPx (float): Height of slice ROI in px.
			clean (bool): Will remove all ROIs from embryo object before creating new ones.
			masterROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI that is supposed to be used as a masterROI.
			bleachedROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI that is supposed to be used to indicate the bleached region.
			rimROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI that is substracted from *Slice*. Should lie withing *Slice*.
			
		Returns:
			list: Updated list of ROIs.
		

		"""
		
		#Clean up if necessary
		if clean:
			self.ROIs=[]
			
		#Add masterROI and bleachedROI to ROIs list if they are given (necessary such that getFreeROIId works)
		if masterROI!=None and masterROI not in self.ROIs:
			self.ROIs.append(masterROI)
		if bleachedROI!=None and bleachedROI not in self.ROIs:
			self.ROIs.append(bleachedROI)
		if rimROI!=None and rimROI not in self.ROIs:
			self.ROIs.append(rimROI)
		
		#Check if sliceHeight is given
		if sliceHeightPx==None:
			sliceHeightPx=self.sliceHeightPx
					
		#Create Slice
		if masterROI!=None:
			sl=masterROI
			self.setMasterROIIdx(self.getROIIdx(sl))
			if sl.getName()!="Slice":
				printWarning("masterROI is not named Slice, this might lead to problems.")
				
		else:
			sl=self.newRadialSliceROI("Slice",self.getFreeROIId(),center,radius,sliceHeightPx,self.sliceWidthPx,self.sliceBottom,color='g',asMaster=True)
		
		#Create Rim
		if rimROI!=None:
			sl2=rimROI
		else:
			sl2=self.newRadialSliceROI("Slice rim",self.getFreeROIId(),center,rimFactor*radius,sliceHeightPx,self.sliceWidthPx,self.sliceBottom,color='y')
		 
		rim=self.newCustomROI("Rim",self.getFreeROIId(),color='m')
		rim.addROI(sl,1)
		rim.addROI(sl2,-1)
		rim.setUseForRim(True)
		
		#Create Bleached Square
		if bleachedROI!=None:
			squ=bleachedROI
			if squ.getName()!="Bleached Square":
				printWarning("Bleached ROI is not named Bleached Square, this might lead to problems.")
		else:	
			squ=self.newSquareSliceROI("Bleached Square",self.getFreeROIId(),self.offsetBleachedPx,self.sideLengthBleachedPx,sliceHeightPx,self.sliceWidthPx,self.sliceBottom,color='b')
		
		return self.ROIs
	
	def getROIs(self):
		
		"""Returns ``ROIs`` list."""
		
		return self.ROIs
	
	def getROIByName(self,name):
		
		"""Returns ROI in ``ROIs`` list with specified name.
		
		If ROI with ``name`` does not exist, returns ``None``.
		"""
		
		for r in self.ROIs:
			if r.name==name:
				return r
		
		printWarning('Cannot find ROI with name '+ name + '. Will return None. This can lead to further problems.')
		
		return None

	def getROIById(self,Id):
		
		"""Returns ROI in ``ROIs`` list with specified ID.
		
		If ROI with ``Id`` does not exist, returns ``None``.
		"""
		
		for r in self.ROIs:
			if r.Id==Id:
				return r
			
		
		printWarning('Cannot find ROI with Id '+ Id + '. Will return None. This can lead to further problems.')
		
		return None
	
	def updateFileList(self):
		
		"""Updates file list containing all names of recovery images.
		
		If new ``fileList`` is not empty, will update number of frames
		to match number of files in ``fileList`` by calling :py:meth:`updateNFrames`.
		
		Returns:
			list: Updated file list.
		
		"""
		
		self.fileList=pyfrp_misc_module.getSortedFileList(self.fnDatafolder,self.dataFT)
		if len(self.fileList)==0:
			printWarning("There are no files of type " + self.dataFT + " in " + self.fnDatafolder +" . This can lead to problems.")
		else:
			self.updateNFrames()
		return self.fileList
	
	def setFileList(self,l):
		
		"""Sets file list to ``l``."""
		
		self.fileList=l
		return self.fileList
	
	def getFileList(self):
		
		"""Returns list of recovery data files."""
		
		return self.fileList
	
	def setDataFolder(self,fn):
		
		"""Set folder containing recovery data files.
		
		Will automatically try to update ``fileList`` by calling
		:py:meth:`updateFileList`.
		
		Args:
			fn (str): Path to folder containing data files.
		
		Returns:
			str: New data folder path.
		
		"""
		
		fn=fn.replace('//','/')
		self.fnDatafolder=fn
		self.updateFileList()
		return self.fnDatafolder
	
	def getDataFolder(self):
		
		"""Returns folder containing recovery data files."""
		
		return self.fnDatafolder
	
	def getGeometry(self):
		
		"""Returns embryo's geometry object.
		
		Returns:
			pyfrp.subclasses.pyfrp_geometry.geometry: PyFRAP geometry object.
		"""
		
		return self.geometry
	
	def setGeometry2ZebraFishDomeStage(self,center,imagingRadius,radiusScale=1.1):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.zebrafishDomeStage`.
		
		Args:
			center (list): Center of geometry.
			imagingRadius (float): Radius of embryo at imaging slice.
			
		Keyword Args:
			radiusScale (float): Scaling factor defining how much bigger outer radius is to inner radius.
			
		Returns:
			pyfrp.subclasses.pyfrp_geometry.zebrafishDomeStage: New zebrafish geometry.
		"""
		
		self.geometry=pyfrp_geometry.zebrafishDomeStage(self,center,imagingRadius,radiusScale=radiusScale)
		return self.geometry
	
	def setGeometry2Cylinder(self,center,radius,height):
			
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.cylinder`.
		
		Args:
			center (list): Center of geometry.
			radius (float): Radius of cylinder.
			height (float): Height of cylinder.
				
		Returns:
			pyfrp.subclasses.pyfrp_geometry.cylinder: New cylinder geometry.
		"""
		
		
		self.geometry=pyfrp_geometry.cylinder(self,center,radius,height)
		return self.geometry
	
	def setGeometry2Cone(self,center,upperRadius,lowerRadius,height):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.cone`.
		
		Args:
			center (list): Center of geometry.
			upperRadius (float): Radius at upper end of cone.
			lowerRadius (float): Radius at lower end of cone.
			height (float): Height of cylinder.
			
		Returns:
			pyfrp.subclasses.pyfrp_geometry.cone: New cone geometry.
		"""
		
		self.geometry=pyfrp_geometry.cone(self,center,upperRadius,lowerRadius,height)
		return self.geometry
	
	def setGeometry2Ball(self,center,imagingRadius):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.ball`.
		
		Args:
			center (list): Center of geometry.
			imagingRadius (float): Radius of embryo in imaging slice.
			
		Returns:
			pyfrp.subclasses.pyfrp_geometry.ball: New ball geometry.
		"""
		
		self.geometry=pyfrp_geometry.xenopusBall(self,center,imagingRadius)
		return self.geometry
	
	def setGeometry2ZebraFishDomeStageQuad(self,center,imagingRadius,radiusScale=1.1):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.zebrafishDomeStageQuad`.
		
		.. warning:: Quadrant reduction is still experimental.
		
		Args:
			center (list): Center of geometry.
			imagingRadius (float): Radius of embryo at imaging slice.
			
		Keyword Args:
			radiusScale (float): Scaling factor defining how much bigger outer radius is to inner radius.
			
		Returns:
			pyfrp.subclasses.pyfrp_geometry.zebrafishDomeStageQuad: New zebrafish quadrant geometry.
		"""
		
		
		self.geometry=pyfrp_geometry.zebrafishDomeStageQuad(self,center,imagingRadius,radiusScale=radiusScale)
		return self.geometry
	
	def setGeometry2CylinderQuad(self,center,radius,height):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.cylinderQuad`.
		
		.. warning:: Quadrant reduction is still experimental.
		
		Args:
			center (list): Center of geometry.
			radius (float): Radius of cylinder.
			height (float): Height of cylinder.
				
		Returns:
			pyfrp.subclasses.pyfrp_geometry.cylinder: New cylinder quadrant geometry.
		"""
		
		self.geometry=pyfrp_geometry.cylinderQuad(self,center,radius,height)
		return self.geometry
	
	def setGeometry2BallQuad(self,center,imagingRadius):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.ballQuad`.
		
		.. warning:: Quadrant reduction is still experimental.
		
		Args:
			center (list): Center of geometry.
			imagingRadius (float): Radius of embryo in imaging slice.
			
		Returns:
			pyfrp.subclasses.pyfrp_geometry.ball: New ball quadrant geometry.
		"""
		
		self.geometry=pyfrp_geometry.xenopusBallQuad(self,center,imagingRadius)
		return self.geometry
	
	def setGeometry2Custom(self,center,fnGeo="",dim=3):
		
		"""Sets embryo's geometry to :py:class:`pyfrp.subclasses.pyfrp_geometry.custom`.
		
		Args:
			center (list): Center of geometry.
			fnGeo (str): Path to geometry file.
			
		Returns:
			pyfrp.subclasses.pyfrp_geometry.custom: New custom geometry.
		"""
		
		self.geometry=pyfrp_geometry.custom(self,center,fnGeo,dim=dim)
		return self.geometry
	
	def newSimulation(self):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_simulation.simulation` object and sets it as
		embryos's ``simulation``.
		
		Returns:
			pyfrp.subclasses.pyfrp_simulation.simulation: New simulation object.
		"""
		
		self.simulation=pyfrp_simulation.simulation(self)
		return self.simulation
	
	def newAnalysis(self):
		
		"""Creates new :py:class:`pyfrp.subclasses.pyfrp_analysis.analysis` object and sets it as
		embryos's ``analysis``.
		
		Returns:
			pyfrp.subclasses.pyfrp_analysis.analysis: New analysis object.
		"""
		
		self.analysis=pyfrp_analysis.analysis(self)
		return self.analysis
	
	def computeROIIdxs(self,signal=None,debug=True):
		
		"""Computes image, extended and mesh indices of all ROIs in embryo's ``ROIs`` list.
		
		Keyword Args:
			signal (PyQt4.QtCore.pyqtSignal): PyQT signal to send progress to GUI.
			debug (bool): Print final debugging messages and show debugging plots.
		
		Returns:
			list: Updated list of ROIs.
		"""
		
		for i,r in enumerate(self.ROIs):
			startInit=time.clock()
			r.computeIdxs()
			
			if debug:
				print r.name, time.clock()-startInit
			if signal:
				signal.emit(int(100.*i)/float(len(self.ROIs)))
			
		return self.ROIs
	
	def geometry2Quad(self):
		
		"""Converts current geometry to quadrant reduced version if available,
		keeping essential parameters the same.
		
		If geometry is already reduced or there is no quadrant version of the geometry,
		will do nothing and return unchanged geometry.
		
		.. warning:: Quadrant reduction is still experimental.
		
		.. note:: Will set ``fnGeo`` back to default value in ``meshfiles`` folder.
	
		Returns:
			pyfrp.subclasses.pyfrp_geometry.geometry: Updated geometry.
		"""
		
		if self.geometry==None:
			printError("No geometry selected yet.")
			return self.geometry	
		
		if 'Quad' in self.geometry.typ:
			printError("Geometry already set to quad.")
			return self.geometry	
		
		if self.geometry.typ=='zebrafishDomeStage':
			self.setGeometry2BallQuad(self,self.geometry.getCenter(),self.geomtry.getImagingRadius())
		elif self.geometry.typ=='cylinder':
			self.setGeometry2CylinderQuad(self,self.geometry.getCenter(),self.geomtry.getRadius())
		elif self.geometry.typ=='xenopusBall':
			self.setGeometry2BallQuad(self,self.geometry.getCenter(),self.geomtry.getImagingRadius())
		else:
			printError("Geometry of type " + self.geometry.typ + " has no quadrant reduced version.") 
		
		
		return self.geometry	
			
	
	def geometry2Full(self):
		
		"""If current geometry was in quadrant reduced version, converts it to 
		full version, keeping essential parameters the same.
		
		.. warning:: Quadrant reduction is still experimental.
		
		.. note:: Will set ``fnGeo`` back to default value in ``meshfiles`` folder.
	
		Returns:
			pyfrp.subclasses.pyfrp_geometry.geometry: Updated geometry.
		"""
		
		if self.geometry==None:
			printError("No geometry selected yet.")
			return
		
		if 'Quad' not in self.geometry.typ:
			printError("Geometry already set to full.")
			return
		
		if self.geometry.typ=='zebrafishDomeStageQuad':
			self.setGeometry2Ball(self,self.geometry.getCenter(),self.geomtry.getImagingRadius())
		elif self.geometry.typ=='cylinderQuad':
			self.setGeometry2Cylinder(self,self.geometry.getCenter(),self.geomtry.getRadius())
		elif self.geometry.typ=='xenopusBallQuad':
			self.setGeometry2Ball(self,self.geometry.getCenter(),self.geomtry.getImagingRadius())
		
		return self.geometry	
	
	def setEmbryo2Quad(self):
		
		"""Reduces both geometry and ROIs of embryo to quadrant, such that 
		embryo object is fully quadrant reduced.
		
		Returns:
			tuple: Tuple Containing:
			
				* ``self.geometry`` (pyfrp.subclasses.pyfrp_geometry.geometry): Updated geometry.
				* ``self.ROIs`` (list): Updated list of ROIs.
		"""
		
		self.geometry2Quad()
		self.ROIs2Quad()
		return self.geometry,self.ROIs
	
	def setEmbryo2Full(self):	
		
		"""Sets both geometry and ROIs to full mode.
		
		Returns:
			tuple: Tuple Containing:
			
				* ``self.geometry`` (pyfrp.subclasses.pyfrp_geometry.geometry): Updated geometry.
				* ``self.ROIs`` (list): Updated list of ROIs.
		"""
		
		self.geometry2Full()
		self.ROIs2Full()
		return self.geometry,self.ROIs
	
	def ROIs2Quad(self):
		
		"""Reduces ROIs to quadrant reduced mode, mapping all their indices in first quadrant.
		
		.. note:: Use :py:func:setEmbryo2Quad to make sure that both geometry and ROIs are reduced.
		
		.. warning:: Quadrant reduction is still experimental.
		
		Returns:
			list: Updated list of ROIs.
		"""
		
		for r in self.ROIs:
			r.idxs2Quad()
		return self.ROIs
	
	def ROIs2Full(self):
		
		"""Sets ROIs to full mode.

		Returns:
			list: Updated list of ROIs.
		"""
		
		for r in self.ROIs:
			r.idxs2Full()
		return self.ROIs
	
	def showAllROIBoundaries(self,ax=None,withImg=False,idx=0):
		
		"""Shows boundaries of all ROIs in ``ROIs`` list.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			ax (matplotlib.axes): Axes to be plotted in.
			withImg (bool): Shows data image inside same axes.
			idx (int): Index of data image to be shown.
			
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		for r in self.ROIs:
			if hasattr(r,'showBoundary'):
				ax=r.showBoundary(ax=ax)
		
		if withImg:
			ax=self.showDataImg(ax=ax,idx=idx)
		
		pyfrp_plot_module.redraw(ax)
		
		return ax
	
	def loadDataImg(self,idx):
		
		"""Loads data image in ``fnDatafolder`` of index ``idx``.
	
		Args:
			idx (int): Index of data image to be loaded.
			
		Returns:
			numpy.ndarray: Loaded image.
		"""
		
		return pyfrp_img_module.loadImg(self.fnDatafolder+self.fileList[idx],self.dataEnc)
	
	def showDataImg(self,ax=None,idx=0):
		
		"""Shows data image in ``fnDatafolder`` of index ``idx``.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			idx (int): Index of data image to be shown.
			
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		if ax==None:
			fig,axes = pyfrp_plot_module.makeSubplot([1,1],sup=["Embryo" + self.name],titles=["DataImg "+str(idx)],tight=False)
			ax=axes[0]
			
		img=self.loadDataImg(idx)	
		ax.imshow(img)
		
		return ax
		
	def showAllROIIdxs(self,axes=None):
		
		"""Shows image, extended and mesh indices of all ROIs in ``ROIs`` list.
		
		If no axes are given via ``axes``, will create new list of matplotlib axes. 
		If axes are given, they should have ``len(axes)=3*len(ROIs)``.
		
		.. note:: If the mesh has a large number of nodes, this can take up a lot
		   of memory due to every node being plotted in a matplotlib scatter plot.
		
		Keyword Args:
			axes (list): List of matplotlib.axes to be plotted in.
			
		Returns:
			matplotlib.axes: List of axes used for plotting.
		
		"""
		
		
		if axes==None:
			proj=len(self.ROIs)*['3d']+len(self.ROIs)*[None]+len(self.ROIs)*[None]
			fig,axes = pyfrp_plot_module.makeSubplot([3,len(self.ROIs)],sup=["Embryo" + self.name + " ROI Indices"],tight=True,proj=proj)
			
		for i,r in enumerate(self.ROIs):
			print r.name
			currAxes=[axes[0+i],axes[len(self.ROIs)+i],axes[2*len(self.ROIs)+i]]
			r.showIdxs(axes=currAxes)
		
		return axes
			
	
	def plotAllData(self,ax=None,legend=True):
		
		"""Plots all data timeseries for all ROIs in ``ROIs`` list.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			ax (matplotlib.axes): Axes to be plotted in.
			legend (bool): Show legend in plot.
			
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		for r in self.ROIs:
			ax=r.plotData(ax=ax,legend=legend)
	
		return ax	
	
	def plotAllSim(self,ax=None,legend=True):
		
		"""Plots all simulation timeseries for all ROIs in ``ROIs`` list.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			ax (matplotlib.axes): Axes to be plotted in.
			legend (bool): Show legend in plot.
			
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		for r in self.ROIs:
			ax=r.plotSim(ax=ax)
		
		if legend:
			ax.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
		
		return ax	
	
	def plotAllDataPinned(self,ax=None,legend=True):
	
		"""Plots all pinned data timeseries for all ROIs in ``ROIs`` list.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			ax (matplotlib.axes): Axes to be plotted in.
			legend (bool): Show legend in plot.
			
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		for r in self.ROIs:
			ax=r.plotDataPinned(ax=ax,legend=legend)
		
		return ax	
	
	def plotAllSimPinned(self,ax=None,legend=True):
		
		"""Plots all pinned simulation timeseries for all ROIs in ``ROIs`` list.
		
		If no axes are given via ``ax``, will create new matplotlib axes.
		
		Keyword Args:
			ax (matplotlib.axes): Axes to be plotted in.
			legend (bool): Show legend in plot.
			
		Returns:
			matplotlib.axes: Axes used for plotting.
		
		"""
		
		for r in self.ROIs:
			ax=r.plotSimPinned(ax=ax,legend=legend)
		
		if legend:
			ax.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
		
		return ax	
	
	
	def checkQuadReducable(self,tryFix=False,auto=False,debug=False):
		
		"""Checks if embryo is reducable to quadrant by checking if all
		ROIs are either point or axis symmetric around geometry center.
		
		.. note:: You want to call :py:meth:showAllROIBoundaries and 
		   :py:meth:computeROIIdxs afterwards to make sure everything went
		   properly.
		   
		.. warning:: Quadrant reduction is still experimental.   
		   
		Keyword Args:
			tryFix (bool): Tries to readjust ROIs into reducable form.
			auto (bool): Readjust ROIs automatically.
			debug (bool): Print debugging messages.
			
		Returns:
			bool: True if embryo is reducable.
		
		"""
		
		reducable=True
		for r in self.ROIs:
			if not r.checkSymmetry(debug=debug):
				if tryFix:
					if hasattr(r,'makeReducable'):
						reducable=r.makeReducable(auto=auto,debug=debug)
				else:		
					printWarning('Cannot reduce region '+ self.name+' to quadrant. Indices are not symmetric.')
					reducable=False
				
		return reducable		
	
	def makeQuadReducable(self,auto=False,debug=False):
		
		"""Makes embryo quadrant reducable by:
		
			* Checks if embryo is reducable to quadrant by checking if all
			  ROIs are either point or axis symmetric around geometry center.
			* Tries to readjust non-symmetric ROIs to make them quad-reducable.
			* If ROIs are reducable, will center geometry at center of data image.
		
		.. note:: You want to call :py:meth:showAllROIBoundaries and 
		   :py:meth:computeROIIdxs afterwards to make sure everything went
		   properly.
		   
		.. warning:: Quadrant reduction is still experimental.   
		   
		Keyword Args:
			tryFix (bool): Tries to readjust ROIs into reducable form.
			auto (bool): Readjust ROIs automatically.
			debug (bool): Print debugging messages.
			
		Returns:
			bool: True if embryo is reducable.
		
		"""
		
		reducable=self.checkQuadReducable(tryFix=True,auto=auto,debug=debug)

		if reducable:
			self.geometry.centerInImg()
		return reducable
	
	def getMasterROI(self):
		
		"""Returns master ROI."""
		
		return self.ROIs[self.masterROIIdx]
	
	def getOptimalAllROI(self,name='All',makeNew=False):
		
		"""Readjusts ROI with name *All*  (if existent) to cover whole geometry."""
		
		self.geometry.setAllROI(name=name,makeNew=makeNew)
	
	def computePinVals(self,useMin=True,useMax=True,bkgdVal=None,debug=False):
		
		"""Compute overall pinning values over all ROIs.
		
		Keyword Args:
			useMin (bool): Use minimum value for background computation.
			useMax (bool): Use maximum value for norm value computation.
			bkgdVal (float): Use this background value instead of newly computing it.
			debug (bool): Print debugging messages.
	
		Returns:
			tuple: Tuple containing:
				
				* bkgdVal (float): Background value.
				* normVal (float): Norming value.
			
		"""
		
		if bkgdVal==None:
			bkgdVal=self.computeBkgd(useMin=useMin,debug=debug)
		normVal=self.computeNorm(bkgdVal,useMax=useMax,debug=debug)
		
		return bkgdVal,normVal
	
	def computeBkgd(self,useMin=False,fromTS='both',debug=False):
		
		"""Computes background value over all ROIs.
		
		If ``useMin==False``, will use value at first index of data/simulation
		vectors as norming value.
		
		.. note:: Use ``fromTS='both'`` to use values from both simulation and 
		   data for background computation. If ``fromTS='data'``, will only use 
		   data, of ``fromTS='sim'`` will only use simulation vectors.
		
		Keyword Args:
			useMin (bool): Use minimum value for background computation.
			fromTS (bool): Which time series to use for background computation.
			debug (bool): Print debugging messages.
		
		Returns:
			float: Background value. 
			
		"""
		
		bkgds=[]
		for r in self.ROIs:
			if fromTS in ['data','both']:
				if r.isAnalyzed():
					bkgdTemp,norm=pyfrp_fit_module.computePinVals(r.dataVec,useMin=useMin,useMax=False,debug=debug)
					bkgds.append(bkgdTemp)
				else:
					if debug:
						printWarning("ROI " + r.name + " is not analyzed yet, cannot use it for background computation.")
						
			if fromTS in ['sim','both']:
				if r.isSimulated():
					bkgdTemp,norm=pyfrp_fit_module.computePinVals(r.simVec,useMin=useMin,useMax=False,debug=debug)
					bkgds.append(bkgdTemp)
				else:
					if debug:
						printWarning("ROI " + r.name + " is not simulated yet, cannot use it for background computation.")
		
		if len(bkgds)==0:
			if debug:
				printWarning("No bkgd value could be found. Will return 0.")
			return 0.
		
		return min(bkgds)	
	
	def computeNorm(self,bkgdVal,useMax=True,fromTS='both',debug=False):
		
		"""Computes background value over all ROIs.
		
		If ``useMax==False``, will use value at last index of data/simulation
		vectors as norming value.
	
		.. note:: Use ``fromTS='both'`` to use values from both simulation and 
		   data for background computation. If ``fromTS='data'``, will only use 
		   data, of ``fromTS='sim'`` will only use simulation vectors.
		
		Args:
			bkgdVal (float): Use this background value instead of newly computing it.
		
		Keyword Args:
			useMax (bool): Use maximum value for norm value computation.
			fromTS (bool): Which time series to use for background computation.
			debug (bool): Print debugging messages.
		
		Returns:
			float: Norming value. 
			
		"""
		
		norms=[]
		for r in self.ROIs:
			if fromTS in ['data','both']:
				if r.isAnalyzed():
					bkgdTemp,norm=pyfrp_fit_module.computePinVals(r.dataVec,bkgdVal=bkgdVal,useMin=False,useMax=useMax,debug=debug)
					norms.append(norm)
				else:
					if debug:
						printWarning("ROI " + r.name + " is not analyzed yet, cannot use it for norming computation.")
						
			if fromTS in ['sim','both']:
				if r.isSimulated():
					bkgdTemp,norm=pyfrp_fit_module.computePinVals(r.simVec,bkgdVal=bkgdVal,useMin=False,useMax=useMax,debug=debug)
					norms.append(norm)
				else:
					if debug:
						printWarning("ROI " + r.name + " is not simulated yet, cannot use it for norming computation.")
		if len(norms)==0:
			if debug:
				printWarning("No norm value could be found. Will return 1.")
			return 1.
			
		return max(norms)	
			
	def computeIdealFRAPPinVals(self,bkgdName='Bleached Square',normName='Slice',debug=False,useMin=False,useMax=False,sepSim=True,switchThresh=0.95):
		
		"""Computes background and norming value using optimized settings.
		
		Idea: Instead using values from all ROIs to compute pinning values, select 
		two ROIs that should lead to optimal pinning values. In the default case, this
		is the ROI describing the bleached region (here we expect the lowest intensities),
		and the slice (here we expect the overall end concentration the experiment is converging to).
		
		If ``useMin==False``, will use value at first index of data/simulation
		vectors as norming value.
			
		If ``useMax==False``, will use value at last index of data/simulation
		vectors as norming value.
	
		.. note:: If ``bkgdName`` and ``normName`` are not set differently, will look for ROIs
		   with these names for background and norming computation, respectively. If they don't exist,
		   will return ``None``. 
		   :py:meth:`genDefaultROIs` will make sure that both those ROIs exist.
		   
		.. warning:: Not all ROIs are suitable for pinning value computation. Generally ROIs that have the
		   least extended volume proof most suitable.
		
		.. note:: ``switchThresh`` checks if recovery is complete. This is important if recovery curves are 
		   very slow and bleached region does not reach full recovery. If this happens, we divide by a number 
		   that is smaller than 1 and hence boost all timeseries way above one instead of limiting it below one
		
		.. warning:: ``sepSim==True`` makes sure that we never get negative intensities in the pinned simulation 
		   vectors. Since interpolation is never perfect, this can happen if :math:`bkgdValue(data)>bkgdValue(sim)`.
		
		Keyword Args:
			bkgdName (str): Name of ROI used for background computation.
			normName (str): Name of ROI used for norming computation.
			useMin (bool): Use minimum value for background computation.
			useMax (bool): Use maximum value for norm value computation.
			fromTS (bool): Which time series to use for background computation.
			debug (bool): Print debugging messages.
			sepSim (bool): Use seperate pinning values for simulation vectors.
		
		Returns:
			tuple: Tuple containing:
				
				* bkgdVal (float): Background value for data vectors.
				* normVal (float): Norming value for data vectors.
				* bkgdValSim (float): Background value for simulation vectors.
				* normValSim (float): Norming value for simulation vectors.
		
		"""
		
		bkgdROI=self.getROIByName(bkgdName)
		normROI=self.getROIByName(normName)
		
		if bkgdROI==None or normROI==None:
			return None,None,None,None
		
		bkgdValSim=None
		normValSim=None
		
		bkgds=[]
		norms=[]
		
		bkgdBdata,normBdata=pyfrp_fit_module.computePinVals(bkgdROI.dataVec,useMin=useMin,useMax=useMax,debug=debug)
		bkgdBsim,normBsim=pyfrp_fit_module.computePinVals(bkgdROI.simVec,useMin=useMin,useMax=useMax,debug=debug)
		
		bkgdVal=min([bkgdBdata,bkgdBsim])
		
		if sepSim:
			bkgdValSim=bkgdBsim
			bkgdVal=bkgdBdata
		
		"""Need to check if we need to switch ROI. This is important if recovery curves are very slow and bleached region
		does not reach full recovery. If this happens, we divide by a number that is smaller than 1 and hence boost all 
		timeseries way above one instead of limiting it below one."""
		
		if normBdata<switchThresh or normBsim<switchThresh:
			bkgdNdata,normNdata=pyfrp_fit_module.computePinVals(normROI.dataVec,useMin=useMin,useMax=useMax,bkgdVal=bkgdVal,debug=debug)
			bkgdNsim,normNsim=pyfrp_fit_module.computePinVals(normROI.simVec,useMin=useMin,useMax=useMax,bkgdVal=bkgdVal,debug=debug)
			
			normVal=max([normNsim,normNdata])
			if sepSim:
				normValSim=normNsim
				normVal=normNdata
			
			print "in switch"
			
			if debug:
				printNote('Switched to region ' +  normName + ' for computation of normalization value.' )
		else:	
			
			print normBdata+bkgdBdata, normBsim+bkgdBsim
			
			print "not in switch"
			normVal=max([normBdata,normBsim])
			if sepSim:
				normValSim=normBsim
				normVal=normBdata
		
		return bkgdVal, normVal, bkgdValSim, normValSim
		
	def pinAllROIs(self,bkgdVal=None,normVal=None,bkgdValSim=None,normValSim=None,useMin=False,useMax=False,debug=False):
		
		"""Pins both simulation and data vectors of all ROIs.
		
		.. note:: If no bkgdVal or normVal is giving, will try to compute it using
		   :py:meth:`computePinVals`. Only then input of ``useMax`` and ``useMin``
		   are relevant.
		
		Keyword Args:
			bkgdVal (float): Background value used for data pinning.
			normVal (float): Norming value used for data pinning.
			bkgdValSim (float): Background value used for simulation pinning.
			normValSim (float): Norming value used for simulation pinning.
			useMin (bool): Use minimum value for background computation.
			useMax (bool): Use maximum value for norm value computation.
			debug (bool): Print debugging messages.
			
		Returns:
			list: Updated list of ROIs.
		
		"""
		
		bkgdValTemp,normValTemp = self.computePinVals(useMin=useMin,useMax=useMax,debug=debug)
		
		bkgdVal = pyfrp_misc_module.assignIfVal(bkgdVal,bkgdValTemp,None)
		normVal = pyfrp_misc_module.assignIfVal(normVal,normValTemp,None)
		
		for r in self.ROIs:
			r.pinAllTS(bkgdVal=bkgdVal,normVal=normVal,bkgdValSim=bkgdValSim,normValSim=normValSim,debug=debug)
		return self.ROIs	
			
	def printAllAttr(self,full=False):
		
		"""Prints out all attributes of embryo object.""" 
		
		printAllObjAttr(self)
		
		if full:
			self.geometry.printDetails()
			self.analysis.printAllAttr()
			self.simulation.printAllAttr()
			self.simulation.mesh.printAllAttr()
				
		
	def isAnalyzed(self):
		
		"""Returns ``True`` if all ROIs have been analyzed."""
		
		b=True
		for r in self.ROIs:
			b=b and r.isAnalyzed()
		return b

	def isSimulated(self):
		
		"""Returns ``True`` if all ROIs have been simulated."""
		
		b=True
		for r in self.ROIs:
			b=b and r.isSimulated()
		return b
	
	def isFitted(self):
		
		"""Returns ``True`` if all fits are fitted."""
		
		b=True
		for fit in self.fits:
			b=b and fit.isFitted()
		return b	
			
	def getFitByName(self,name):
		
		"""Returns fit in ``fits`` list with name ``name``. 
		
		If it doesn't exists, returns ``None``.
		"""
		
		for fit in self.fits:
			if fit.name==name:
				return fit
			
		printWarning('Cannot find fit with name '+ name + '. Will return None. This can lead to further problems.')
		
		return None
	
	def getInterpolationError(self):
		
		"""Prints out interpolation error by ROI.
		"""
		
		for r in self.ROIs:
			print r.name, r.getInterpolationError()
	
	def checkROIIdxs(self,debug=False):
		
		"""Checks if all ROIs have their mesh and image indices computed.
		
		Keyword Args:
			debug (bool): Print debugging messages.
		
		Returns:
			tuple: Tuple containing:
			
				* img (bool): True if all ROIs have up-to-date image indices.
				* mesh (bool): True if all ROIs have up-to-date mesh indices.
		"""
		
		img=True
		mesh=True
		for r in self.ROIs:
			if len(r.imgIdxX)==0:
				if debug:
					printWarning("ROI " + r.name+ " needs its img indices computed. Will not be able to perform image analysis otherwise.")
				img=False
			if len(r.meshIdx)==0:
				if debug:
					printWarning("ROI " + r.name+ " needs its mesh indices computed. Will not be able to perform simulation otherwise.")
				mesh=False
		return img,mesh
	
	def quickAnalysis(self,maxDExpPx=None,timeScale='log',runIdxs=True,runAnalysis=True,runSimulation=True,runPin=True,runFit=True,bkgdName='Bleached Square',normName='Slice',useMin=False,useMax=False,sepSim=True,switchThresh=0.95):
			
		"""Performs complete FRAP analysis of embryo object including:
		
			* Finding ROI indices by calling :py:meth:`computeROIIdxs` .
			* Running image analysis through :py:meth:`pyfrp.subclasses.pyfrp_analysis.analysis.run` .
			* Generating optimal time simulation vector through  :py:meth:`pyfrp.subclasses.pyfrp_simulation.simulation.getOptTvecSim`
			* Running simulation through :py:meth:`pyfrp.subclasses.pyfrp_simulation.simulation.run` .
			* Pin simulation and data timeseries using :py:meth:`computeIdealFRAPPinVals` and :py:meth:`pinAllROIs`.
			* Run all fits in ``fits`` list by calling :py:meth:`pyfrp.subclasses.pyfrp_fit.fit.run`
			
		Keyword Args:
			maxDExpPx (float): Maximum expected diffusion coefficient.
			timeScale (str): Linear (``'lin'``) or logarithmic (``'log'``) time scaling.
			runIdxs (bool): Run indexing.
			runAnalysis (bool): Run image analysis.
			runSimulation (bool): Run simulation.
			runPin (bool): Run pinning.
			runFit (bool): Run fitting.
			bkgdName (str): Name of ROI used for background computation.
			normName (str): Name of ROI used for norming computation.
			useMin (bool): Use minimum value for background computation.
			useMax (bool): Use maximum value for norm value computation.
			sepSim (bool): Use seperate pinning values for simulation vectors.
			
			
		"""
		
		if runIdxs:
			self.computeROIIdxs()
		
		#Image analysis
		if runAnalysis:
			self.analysis.run(showProgress=True)

		#Simulation
		if maxDExpPx!=None:
			self.simulation.getOptTvecSim(maxDExpPx)
		
		if timeScale=='log':
			self.simulation.toLogTimeScale()
		
		if runSimulation:
			self.simulation.run(showProgress=True)

		#Pin Concentrations
		if runPin:
			bkgdVal,normVal,bkgdValSim,normValSim=self.computeIdealFRAPPinVals(debug=True,useMin=False,useMax=False,switchThresh=0.95,bkgdName=bkgdName,normName=normName)
			self.pinAllROIs(bkgdVal=bkgdVal,normVal=normVal,bkgdValSim=bkgdValSim,normValSim=normValSim,debug=False)

		#Run all fits
		if runFit:
			for fit in self.fits:
				fit.run(debug=False)
			
		
	def clearAllAttributes(self):
		
		"""Replaces all attribute values of embryo object with ``None``, except ``name``.
		
		Useful if embryos are seperated and molecule file needs to be compressed.
		
		Returns:
			bool: True if success, False else.
		
		"""
		
		try:
			for item in vars(self):
				if item!="name":
					vars(self)[str(item)]=None
			return True				
		except:
			printError("Failed to clearAllAttributes in embryo" + self.name +" .")
			return False
				
	def sliceEmbryo(self,nSlices,direction='z'):
		
		"""Slices embryo in z-direction in ``nSlices``.
		
		Creates ``nSlices`` new :py:class:`pyfrp.subclasses.pyfrp_ROI.sliceROI` ROIs
		and appends them to ``ROIs`` list.
		
		.. note:: Slice ROIs will be created with ``sliceBottom=False``.
		
		Args:
			nSlices (int): Number of slices embryo is supposed to get cut.
		
		Returns:
			list: Updates ``ROIs`` list.
			
		"""
		
		if direction=='z':
			
			#Get zExtend and height of geometry
			zmin,zmax = self.geometry.getZExtend()
			height=abs(zmax-zmin)
			
			sliceWidth = height/nSlices
			
			for i in range(nSlices):
				self.newSliceROI("Slice"+str(i),self.getFreeROIId(),zmin+(i+0.5)*sliceWidth,sliceWidth,False,color=[1,1/(i+2.),0.])
		
		elif direction=='x':
			xmin,xmax,ymin,ymax = self.geometry.getXYExtend()
			xwidth=abs(xmax-xmin)
			ywidth=abs(ymax-ymin)
			sliceWidth = xwidth/nSlices
			
			for i in range(nSlices):
				self.newRectangleROI("Slice"+str(i),self.getFreeROIId(),[xmin+i*sliceWidth,ymin],sliceWidth,ywidth,color=[1,1/(i+2.),0.])
				
		elif direction=='y':
			xmin,xmax,ymin,ymax = self.geometry.getXYExtend()
			xwidth=abs(xmax-xmin)
			ywidth=abs(ymax-ymin)
			sliceWidth = ywidth/nSlices
			
			for i in range(nSlices):
				self.newRectangleROI("Slice"+str(i),self.getFreeROIId(),[xmin,ymin+i*sliceWidth],xwidth,sliceWidth,color=[1,1/(i+2.),0.])
				
				
		return self.ROIs
	
	
	def compareFitsByAIC(self,ROIs=None,sigma=1,fromSSD=True,thresh=None,printOut=True):
		
		"""Compares all fits of embryo using the Akaike information criterion (AIC).
		
		For a detailed explanation of the model selection procedure, please refer to
		:py:func:`pyfrp.modules.pyfrp_stats_module.compareFitsByAIC`.
		
		If ``printOut`` is selected, will print a list of selected fits and the 
		underlying Akaike values in a readable table.
		
		If the AIC or the AICc should be used can be determined using 
		:py:func:`pyfrp.modules.pyfrp_stats_module.useAIC`.
		
		Keyword Args:	
			ROIs (list): List of ROIs to be considered for computation.
			sigma (float): Standard deviation of normal distribution if known.
			fromSSD (bool): Simply use SSD as maximum likelihood.
			thresh (float): Probability range for model selection.
			
		Returns:
			tuple: Tuple containing:
			
				* AICs (list): List of AIC values of the respective fits.
				* deltaAICs (numpy.ndarray): List of Akaike difference values of the respective fits.
				* weights (numpy.ndarray): List of Akaike difference weights of the respective fits.
				* acc (list): List of acceptable fits by model selection.
				* ks (list): List of number of parameters fitted of the respective fits.
				* ns (list): List of number of datapoints fitted of the respective fits.
		"""
		
		AICs, deltaAICs,weights,acc,ks,ns = pyfrp_stats_module.compareFitsByAIC(self.fits,ROIs=ROIs,sigma=sigma,fromSSD=fromSSD,thresh=thresh)
		
		
		if printOut:
			fitNames = pyfrp_misc_module.objAttrToList(self.fits,'name')
			printNote("Acceptable fits under threshold "+ str(thresh) +":")
			for fit in acc:
				print fit.name
			
			print "Details of AIC analysis can be found in the following table:"
			printTable([fitNames,AICs,deltaAICs,weights,ks,ns],["fitNames","AICs","deltaAICs","weights","k","n"],col=True)
			
		return AICs, deltaAICs,weights, acc,ks,ns
			
	def compareFitsByCorrAIC(self,ROIs=None,sigma=1,fromSSD=True,thresh=None,printOut=True):
		
		"""Compares all fits of embryo using the corrected Akaike information criterion (AICc).
		
		For a detailed explanation of the model selection procedure, please refer to
		:py:func:`pyfrp.modules.pyfrp_stats_module.compareFitsByCorrAIC`.
		
		If ``printOut`` is selected, will print a list of selected fits and the 
		underlying Akaike values in a readable table.
		
		If the AIC or the AICc should be used can be determined using 
		:py:func:`pyfrp.modules.pyfrp_stats_module.useAIC`.
		
		Keyword Args:	
			ROIs (list): List of ROIs to be considered for computation.
			sigma (float): Standard deviation of normal distribution if known.
			fromSSD (bool): Simply use SSD as maximum likelihood.
			thresh (float): Probability range for model selection.
			
		Returns:
			tuple: Tuple containing:
			
				* AICs (list): List of AICc values of the respective fits.
				* deltaAICs (numpy.ndarray): List of Akaike difference values of the respective fits.
				* weights (numpy.ndarray): List of Akaike difference weights of the respective fits.
				* acc (list): List of acceptable fits by model selection.
				* ks (list): List of number of parameters fitted of the respective fits.
				* ns (list): List of number of datapoints fitted of the respective fits.
		
		"""
		
		AICs, deltaAICs,weights,acc,ks,ns = pyfrp_stats_module.compareFitsByCorrAIC(self.fits,ROIs=ROIs,sigma=sigma,fromSSD=fromSSD,thresh=thresh)
		
		if printOut:
			fitNames = pyfrp_misc_module.objAttrToList(self.fits,'name')
			printNote("Acceptable fits under threshold "+ str(thresh) +":")
			for fit in acc:
				print fit.name
			
			print "Details of AIC analysis can be found in the following table:"
			printTable([fitNames,AICs,deltaAICs,weights,ks,ns],["fitNames","AICcs","deltaAICs","weights","k","n"],col=True)
		
		return AICs, deltaAICs,weights, acc,ks,ns	
	
	def getRimFactorByPx(self,radius,px):
		
		r"""Returns the correct rimFactor if a rim of with ``px`` in a 
		ROI of radius ``radius`` is desired.
		
		Rim factor :math:`f` is then given by:
		
		.. math:: f=1-\frac{p}{r},
		
		where :math:`p` is ``px`` and :math:`r` is ``radius``.
		
		Args:
			radius (float): Radius of ROI.
			px (float): Number of pixels that rim should be wide.
			
		Returns:
			float: Calculated rim factor.
		
		"""
		
		return 1-px/radius
	
	def fixFilePaths(self):
		
		"""Fixes paths to geometry/meshfiles.
		
		Returns:
			bool: True if success for all paths.
			
		
		"""
		
		b=True
		try:
			self.geomtry.fnGeo=pyfrp_misc_module.fixPath(self.geomtry.fnGeo)	
		except:
			b=False
		try:
			self.simulation.mesh.fnMesh=pyfrp_misc_module.fixPath(self.geomtry.fnMesh)	
		except:
			b=False
			
		return b
		
		
	
	
	#def grabDataDetails(self):
	###NOTE make a function that automatically grabs filetype and what not			


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Essential PyFRAP module containing :py:class:`mesh` class. 
"""

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================


#PyFRAP
from pyfrp.modules import pyfrp_gmsh_module
from pyfrp.modules import pyfrp_gmsh_IO_module
from pyfrp.modules import pyfrp_integration_module
from pyfrp.modules import pyfrp_plot_module
from pyfrp.modules import pyfrp_misc_module
from pyfrp.modules import pyfrp_gmsh_geometry
from pyfrp.modules.pyfrp_term_module import *

#FiPy
import fipy

#Numpy/Scipy
import numpy as np

#Misc
import os
import os.path


#===========================================================================================================================================================================
#Class definitions
#===========================================================================================================================================================================

class mesh(object):
	
	"""Mesh class for PyFRAP.
	
	The mesh class stores all information about location and creation of the mesh used for a simulation. 
	It is directly associated with the :py:class:`pyfrp.subclasses.pyfrp_simulation.simulation` object
	that uses it.
	
	Meshes can either be created via running Gmsh onto the *.geo* file of the :py:class:`pyfrp.subclasses.pyfrp_geometry.geometry`,
	or by running Gmsh internally from FiPy using some predefined functions (limited geometry support). See also 
	:py:func:`genMesh`.
	
	The most important attributes are:
	
		* ``mesh``: The actual mesh as a ``fipy.GmshImporter3D`` object.
		* ``fromFile``: Flag that controls if mesh should be created from *.geo* file or not.
		* ``volSizePx``: Mesh element size in px.
	
	Besides mesh storage and creation, the mesh class contains useful functions such as:
		
		* Mesh refinement, see :py:func:`refine`, :py:func:`addBoxField` and :py:func:`forceMinMeshDensityInROI`.
		* Information output, see :py:func:`printStats`.
		* Plotting, see :py:func:`plotMesh` and :py:func:`plotDensity`.
	
	Args:
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
	
	"""

	def __init__(self,simulation):
		
		#Naming/ID
		self.simulation=simulation
		self.mesh=None
		self.restoreDefaults()
	
	def setVolSizePx(self,v,remesh=True,fnOut=None):
		
		"""Sets volSize of mesh in px.
		
		.. note:: If ``fnOut=None``, then either ``fnMesh`` will be used, or,
		   if ``fnMesh`` is not set yet, will use ``geometry.fnGeo``.
		
		Args:
			v (float): New volSize.
			
		Keyword Args:
			remesh (bool): Generate mesh with new volSize.
			fnOut (str): Output filepath for meshfile.
			dim (int): Dimension of mesh.
		
		Returns:
			float: New volSize.
		
		"""
		
		
		self.volSizePx=v
		self.updateGeoFile()
		if remesh:
			self.fnMesh=self.genMesh(fnOut=fnOut)
		return self.volSizePx
	
	def getVolSizePx(self):
		
		"""Returns mesh volSize in px.
		
		Returns:
			float: VolSize.
		
		"""
		
		return self.volSizePx
		
	def getSimulation(self):
		
		"""Returns :py:class:`pyfrp.subclasses.pyfrp_simulation` that mesh belongs to.
		
		Returns:
			pyfrp.subclasses.pyfrp_simulation: Simulation object.
		"""
		
		return self.simulation
	
	def restoreDefaults(self):	
		
		"""Restores default parameters of mesh.
		
		Default parameters are:
		
			* ``mesh.geometry=mesh.simulation.embryo.geometry``
			* ``mesh.fromFile=True``
			* ``mesh.volSizePx=20``
			* ``mesh.fnMesh=""``
	
		"""
		
		#if self.simulation.embryo.geometry==None:
			#printWarning("Geometry of embryo not specified, mesh generation will not work!")
		
		
		self.fromFile=True
		self.volSizePx=20.
		self.fnMesh=""
		
	def genMesh(self,fnOut=None,debug=False):
		
		"""Main mesh generation function.
		
		.. note:: If ``fnOut=None``, will use ``geometry.fnGeo``.
		
		.. note:: If ``fromFile=True``, will generate from .geo file running
		   gmsh directly on the file. If not, will try to run hard coded FiPy
		   version for mesh generation via :py:func:`runFiPyMeshGenerator` .
		
		Keyword Args:
			fnOut (str): Output filepath for meshfile.
			debug (bool): Print debugging messages.
			dim (int): Dimension of mesh.
			
		Returns:
			fipy.GmshImporter3D: Gmsh mesh object.
		
		"""
		
		if fnOut==None:
			fnOut=self.simulation.embryo.geometry.fnGeo.replace(".geo",".msh")
		
		dim=self.simulation.embryo.geometry.getDim()
		
		if self.fromFile:
			self.fnMesh=pyfrp_misc_module.fixPath(fnOut)
		
			pyfrp_gmsh_module.runGmsh(self.simulation.embryo.geometry.fnGeo,fnOut=fnOut,debug=debug,volSizeMax=self.volSizePx,dim=dim)
			
			self.importMeshFromFile(self.fnMesh)
		else:
			self.runFiPyMeshGenerator(self.simulation.embryo.geometry.typ)

		return self.mesh
	
	def runFiPyMeshGenerator(self,typ):
		
		"""Runs gmsh on the via FiPy internally defined meshes.
		
		Available meshes:
			
			* cylinder
			* zebrafishDomeStage
			* xenopusBall
			
		.. note:: Any refinement method will not work if mesh is created this way.
		
		Args:
			typ (str): Type of mesh to be created (see list above).
				
		Returns:
			fipy.GmshImporter3D: Gmsh mesh object.
			
		"""
		
		if typ=="cylinder":
			self.mesh=pyfrp_gmsh_module.genFiPyCylinderMesh(self.volSizePx,self.simulation.embryo.geometry.radius,self.simulation.embryo.geometry.height,self.simulation.embryo.geometry.center)
		elif typ=="zebrafishDomeStage":
			self.mesh=pyfrp_gmsh_module.genFiPyDomeMesh(self.volSizePx,self.simulation.embryo.geometry.innerRadius,self.simulation.embryo.geometry.outerRadius,self.simulation.embryo.geometry.centerDist,self.simulation.embryo.geometry.center)
		elif typ=="xenopusBall":
			self.mesh=pyfrp_gmsh_module.genFiPyBallMesh(self.volSizePx,self.simulation.embryo.geometry.radius,self.simulation.embryo.geometry.center)
		else:
			printWarning("Geometry type " + typ + "unknown for runFiPyMeshGenerator. Check pyfrp_gmsh_module for available geometries.")
		
		return self.mesh
		
	def importMeshFromFile(self,fn):
		
		"""Imports mesh from a Gmsh .msh file.
		
		See also http://www.ctcms.nist.gov/fipy/fipy/generated/fipy.meshes.html. 
		
		Args:
			fn (str): Filepath to meshfile.
		
		Keyword Args:
			dim (int): Dimension of mesh.
		
		Returns:
			fipy.GmshImporter3D: Gmsh mesh object.
			
		"""
		
		try:
			dim=self.simulation.embryo.geometry.getDim()
		except AttributeError:
			printWarning("Was not able to receive geometry's dimension. Will assume dim=3.")
			dim=3 
			
		if dim==3:
			self.mesh=fipy.GmshImporter3D(fn)
		elif dim==2:
			self.mesh=fipy.GmshImporter2D(fn)
		else:
			printError("Unknown dimensionality dim = "+str(dim))
		self.fnMesh=fn
		return self.mesh
		
	def setMesh(self,m):
		
		"""Sets ``mesh`` attribute to a new mesh.
		
		Args:
			m (fipy.GmshImporter3D): New mesh.
				
		Returns:
			fipy.GmshImporter3D: Gmsh mesh object.
		
		"""
		
		self.mesh=m
		return self.mesh
	
	def setFromFile(self,v):
		
		"""Sets flag if mesh is supposed to be created from file (recommended)
		or from internally defined mesh creation method.
		
		Args:
			v (bool): New flag value.
				
		Returns:
			bool: New flag value.
		
		"""
		
		self.fromFile=v
		return self.fromFile
	
	def getMesh(self):
		
		"""Returns mesh that is used for simulation.
		
		Returns:
			fipy.GmshImporter3D: Gmsh mesh object.
		
		"""

		return self.mesh
	
	def getFnMesh(self):
		
		"""Returns the filepath of meshfile.
		"""
		
		return self.fnMesh
	
	def setFnMesh(self,fn):
		
		
		"""Sets the filepath of meshfile.
		
		Imports the new mesh right away using :py:func:`importMeshFromFile`.
		
		"""
		
		
		self.fnMesh=pyfrp_misc_module.fixPath(fn)
		self.importMeshFromFile(self.fnMesh)
		return self.fnMesh
	
	def updateGeoFile(self,debug=False):
		
		"""Updates geometry file by writing new ``volSizePx``
		into ``embryo.geometry.fnGeo``.
		
		Keyword Args:
			debug (bool): Print debugging messages.
		
		Returns:
			str: Path to ouput meshfile.
		
		"""
		
		if os.path.isfile(self.simulation.embryo.geometry.fnGeo):
			self.fnMesh=pyfrp_gmsh_module.updateVolSizeGeo(self.simulation.embryo.geometry.fnGeo,self.volSizePx,run=False,debug=debug)
		else:
			printWarning("Cannot update meshfile, since geometry.fnGeo does not exist or is invalid.")
		return self.fnMesh
	
	def refine(self,debug=False):
		
		"""Refines mesh by splitting.
		
		See also http://gmsh.info/doc/texinfo/gmsh.html .
		
		Keyword Args:
			debug (bool): Print debugging messages.
		
		"""
		
		pyfrp_gmsh_module.refineMsh(self.fnMesh,debug=debug)
		self.importMeshFromFile(self.fnMesh)
		return self.fnMesh
	
	def forceMinMeshDensityInROI(self,ROI,density,stepPercentage=0.1,debug=False,findIdxs=True,method='refine',maxCells=100000):
		
		"""Forces global mensh density such that a certain density is reached in a
		given ROI.
		
		Tries to achive a mesh density ``density`` in ``ROI`` by globally refining mesh either 
		through decreasing ``volSizePx`` by ``stepPercentage`` percent (``method=volSize``), or 
		by using Gmsh's ``-refine`` option (``method=refine``). If maximum number of cells is 
		exceeded, will use the last mesh that did not exceed ``maxCells``.
		
		Args:
			ROI (pyfrp.subclasses.pyfrp_ROI.ROI): ROI object.
			density (float): Desired density.
			
		Keyword Args:
			stepPercentage (float): If method is ``volSize``, percentage of ``volSize`` decrease.
			method (str): Refinement method (``refine``/``volSize``).
			maxCells (int): Total maximum number of mesh cells allowed.
			findIdxs (bool): Find ROI indices after refinement.
			debug (bool): Print debugging messages.
		
		Returns:
			float: New ``volSizePx``
		
		"""
		
		#Set counter to 0
		j=0
		
		#Loop until mesh density has been found
		while ROI.getMeshDensity()<density:
			
			#Backup old stuff
			volSizeBackup=self.getVolSizePx()
			meshBackup=self.getMesh()
			
			#Refine
			if method=='volSize':
				self.setVolSizePx((1-stepPercentage)*self.getVolSizePx())
			elif method=='refine':
				self.refine()
			else:
				printError("Unknown method: ", method)
				return
			
			#Recompute Idxs
			ROI.computeMeshIdx(self.mesh)
			
			#Debugging output
			if debug:
				if method=='volSize':
					print "Tried volSizePx ", self.getVolSizePx()
				elif method=='refine':
					print "Refinement step, ", j
				
				print "Current density ", ROI.getMeshDensity(), " desired density ", density
				print "Current number of nodes in ROI ", len(ROI.meshIdx)
				print " Mesh now has ", np.shape(self.mesh.x)[0], "cells." 
			
			#Check if we reached maxCells
			if np.shape(self.mesh.x)[0]>maxCells:
				if debug:
					printWarning("Reached maxCells, will keep previous iteration.")
				
				self.mesh=meshBackup
				self.setVolSizePx(volSizeBackup,remesh=False)
				
			#Increment counter
			j=j+1
			
		if debug:
			print "volSizePx = ", self.getVolSizePx(), "is sufficient."
		
		#Recompute idxs for all ROIs
		if findIdxs:
			self.simulation.embryo.computeROIIdxs()
					
		return self.getVolSizePx()	
	
	def getNNodes(self):
		
		"""Returns number of nodes in mesh. 
		
		If no mesh has been generated yet, will return 0.
		
		Returns:
			int: Number of nodes.
		
		"""
		
		if self.mesh==None:
			return 0
		else:
			return len(self.mesh.getCellCenters()[0])
		
	def writeVTKFile(self,fn="",sub=False):
		
		"""Writes mesh into vtk file.
		
		Uses *meshIO* (https://github.com/nschloe/meshio), to convert the mesh saved
		in ``fnMesh`` to a .vtk file. 
		
		If ``sub==True``, will start a seperate subprocess and submit 
		``pyfrp_meshIO_script.py`` to it. This can be sometimes useful, since PyFRAP
		sometimes tends to crash otherwise.
		
		If no output path is given via ``fn``, will use same path as ``fnMesh``.
		
		.. note:: *meshIO* only gets imported inside this function, making PyFRAP running
		   even without the package installed. However, this feature will only run with
		   *meshIO*.
		
		Keyword Args:
			fn (str): Optional output path.
			sub (bool): Subprocess flag.
		
		Returns:
			str: Used output path.
		
		"""
		
		
		if not os.path.isfile(self.fnMesh):
			printWarning("Filepath to meshfile has not been specified yet. Cannot write VTK file.")
		
		if fn=="":
			fn=self.fnMesh.replace('.msh','.vtk')
		
		if sub:
			
			cmd = "python pyfrp_meshIO_script.py "+ self.fnMesh
			
			import shlex
			import subprocess
			args = shlex.split(cmd)
			
			p = subprocess.Popen(args)
			p.wait()
		else:
			#MeshIO
			import meshio
			
			points, cells, point_data, cell_data, field_data = meshio.read(self.fnMesh)
			meshio.write(fn,points,cells,point_data=point_data,cell_data=cell_data,field_data=field_data)
			
		return fn
	
	def importVTKFile(self,fnVTK="",sub=False):
		
		"""Imports a .vtk file into a vtk renderer.
		
		If ``fnVTK`` is not given, will generate .vtk file from meshfile stored in ``fnMesh``
		using :py:func:`writeVTKFile`.
		
		If ``sub==True``, will start a seperate subprocess and submit 
		``pyfrp_meshIO_script.py`` to it. This can be sometimes useful, since PyFRAP
		sometimes tends to crash otherwise.
		
		.. note:: This function imports *vtk*. *vtk* is only necessary in a few functions,
		   hence only imported when needed. This should make PyFRAP more portable.
		
	
		Keyword Args:
			fnVTK (str): Path to input vtk file.
			sub (bool): Subprocess flag.
			
		Returns:
			vtk.vtkRenderer: Renderer object.
		
		"""
		
		#vtk
		from pyfrp.modules import pyfrp_vtk_module
		
		if not os.path.isfile(self.fnMesh):
			printWarning("Filepath to meshfile has not been specified yet. Cannot plot.")
		
		if fnVTK=="":
			fnVTK=self.writeVTKFile(sub=sub)
		
		renderer = pyfrp_vtk_module.importVTKMeshFile(fnVTK,bkgdColor=[1,1,1],color=[0,0,0])
	
		return renderer
	
	def plotMesh(self,fromFile=False,fnVTK="",bkgd=[255,255,255],color=[0,0,0],renderer=None,renderWindow=None):
		
		"""Plots the mesh using VTK.
		
		If ``fnVTK`` is not given, will generate .vtk file from meshfile stored in ``fnMesh``
		using :py:func:`writeVTKFile`.
		
		.. note:: This function imports *vtk*. *vtk* is only necessary in a few functions,
		   hence only imported when needed. This should make PyFRAP more portable.
		
		Keyword Args:
			fnVTK (str): Path to input vtk file.
			bkgd (list): Background color of renderer in RGB values.
			color (list): Color of mesh in RGB values.
			renderer (vtk.vtkRenderer): Some renderer.
			renderWindow (vtk.RenderWindow): Some renderWindow or dummy value.
			
		Returns:
			vtk.vtkRenderer: Renderer object.
		
		"""
		
		from pyfrp.modules import pyfrp_vtk_module
	
		# Import vtk file
		if fromFile:
			renderer=self.importVTKFile(fnVTK=fnVTK)
		
		# Generate everything directly from mesh
		else:
			grid=pyfrp_vtk_module.meshToUnstructeredGrid(self.mesh)
			renderer = pyfrp_vtk_module.unstructedGridToWireframe(grid,bkgdColor=bkgd,color=color,renderer=renderer)
			renderer, renderWindow, renderWindowInteractor = pyfrp_vtk_module.makeVTKCanvas(offScreen=False,bkgd=bkgd,renderer=renderer)
			
			#rendererWindow=renderer.GetRenderWindow()
			#print rendererWindow
			
		# Create renderWindow
		if renderWindow==None:
			renderer, renderWindow, renderWindowInteractor = pyfrp_vtk_module.makeVTKCanvas(offScreen=False,bkgd=bkgd,renderer=renderer)
		
			# This seems to be the key to have a renderer without displaying. However it returns floating 
			# Errors.
			##renderWindow.OffScreenRenderingOn() 
				
		#Start
		renderWindow.GetInteractor().Initialize()
		renderWindow.GetInteractor().Start()
			
			#print rendererWindow
			
		return renderWindow
	
	def saveMeshToPS(self,fnOut,fnVTK="",renderer=None):
		
		"""Saves mesh to postscript file.
		
		Supported extensions are:
		
			* '.ps'  (PostScript)
			* '.eps' (Encapsualted PostScript)
			* '.pdf' (Portable Document Format)
			* '.tex' (LaTeX)
			* '.svg' (Scalable Vector Graphics)
			
		If ``fnVTK`` is not given, will generate .vtk file from meshfile stored in ``fnMesh``
		using :py:func:`writeVTKFile`.
		
		If no ``renderer`` is given, will create one using :py:func:`plotMesh`. 
		
		.. note:: This function imports *vtk*. *vtk* is only necessary in a few functions,
		   hence only imported when needed. This should make PyFRAP more portable.
		
		Some code taken from http://www.programcreek.com/python/example/23102/vtk.vtkGL2PSExporter .
		
		Args:
			fnOut (str): Path to output file.
		
		Keyword Args:
			fnVTK (str): Path to input vtk file.
			renderer (vtk.vtkOpenGLRenderer): Renderer.
			magnification (int): Degree of magnification.
			
		Returns:
			vtk.vtkGL2PSExporter: Exporter object.
		
		"""
		
		from pyfrp.modules import pyfrp_vtk_module
		
		#Plot again if necessary
		if renderer==None:
			renderer=self.plotMesh(fnVTK=fnVTK)
		
		return pyfrp_vtk_module.saveRendererToPS(renderer,fnOut)
	
	def saveMeshToImg(self,fnOut,fnVTK="",renderer=None,magnification=10,show=True):
		
		"""Saves mesh to image file.
		
		Supported extensions are:
			
			* '.ps'  (PostScript)
			* '.eps' (Encapsualted PostScript)
			* '.pdf' (Portable Document Format)
			* '.jpg' (Joint Photographic Experts Group)
			* '.png' (Portable Network Graphics)
			* '.pnm' (Portable Any Map)
			* '.tif' (Tagged Image File Format)
			* '.bmp' (Bitmap Image)
		
		If ``fnVTK`` is not given, will generate .vtk file from meshfile stored in ``fnMesh``
		using :py:func:`writeVTKFile`.
		
		If no ``renderer`` is given, will create one using :py:func:`plotMesh`. 
		
		.. note:: This function imports *vtk*. *vtk* is only necessary in a few functions,
		   hence only imported when needed. This should make PyFRAP more portable.
		
		Some code taken from http://www.programcreek.com/python/example/23102/vtk.vtkGL2PSExporter .
		
		Args:
			fnOut (str): Path to output file.
		
		Keyword Args:
			fnVTK (str): Path to input vtk file.
			renderer (vtk.vtkOpenGLRenderer): Renderer.
			magnification (int): Degree of magnification.
			show (bool): Show vtk render window.
			
		Returns:
			vtk.vtkExporter: Exporter object.
		
		"""
		
		from pyfrp.modules import pyfrp_vtk_module
		
		#Plot again if necessary
		if renderer==None:
			if show:
				rendererWindow=self.plotMesh(fnVTK=fnVTK)
				renderer=rendererWindow.GetRenderers().GetFirstRenderer()
			else:
				renderer=self.importVTKFile(fnVTK=fnVTK)
				
		return pyfrp_vtk_module.saveRendererToImg(renderer,fnOut,magnification=magnification)
	
	def printStats(self,tetLenghts=False):
		
		"""Prints out statistics of mesh.
		
		Also calculates all tetraheder lengths if ``tetLenghts`` is selected. 
		This might take some time depending on mesh size.
		
		Keyword Args:
			tetLenghts (bool): Also calculate and print out tetrahedra sidelengths.
		
		"""
	
		
		x,y,z = self.getCellCenters()
		
		print "-------------------------------------------"
		print "Mesh Statistics:"
		print "-------------------------------------------"
		print "Mesh has ", np.shape(x)[0] , " cells"
		print "Mesh has ", self.mesh._numberOfVertices, " vertices"
		print "Mesh has ", self.mesh.numberOfFaces, " faces"
		print "min x=", min(x), "max x=", max(x)
		print "min y=", min(y), "max y=", max(y)
		print "min z=", min(z), "max z=", max(z)
		print "Maximum cell volume= ", max(self.mesh.getCellVolumes())
		print "Minimum cell volume= ", min(self.mesh.getCellVolumes())
			
		print "Maximum cell volume is", max(self.mesh.getCellVolumes()), "in cell number=", np.argmax(self.mesh.getCellVolumes())
		print "Minimum cell volume is", min(self.mesh.getCellVolumes()), "in cell number=", np.argmin(self.mesh.getCellVolumes())
		print "Average cell volume is", np.mean(self.mesh.getCellVolumes())

		if tetLenghts:
			slsVec=self.calcAllTetSidelenghts()
			print "Average sidelength of tetrahedron in self.mesh:", np.mean(slsVec)
			print "Maximum sidelength of tetrahedron in self.mesh:", max(slsVec)
			print "Minimum sidelength of tetrahedron in self.mesh:", min(slsVec)	
		
		print
		
		return
	
	def calcAllTetSidelenghts(self):
		
		"""Calculates sidelengths of all tetrahedra.
		
		See also :py:func:`pyfrp.modules.pyfrp_integration_module.calcTetSidelengths`.
		
		Returns:
			list: List of all sidelengths.
		"""
		
		#Calculating sidelengths of tetrahedra
		slsVec=[]
		for i in range(np.shape(self.mesh._getOrderedCellVertexIDs())[1]):
			currVert=self.mesh._getOrderedCellVertexIDs()[:,i]
			
			pt1=self.mesh.vertexCoords[:,currVert[0]]
			pt2=self.mesh.vertexCoords[:,currVert[1]]
			pt3=self.mesh.vertexCoords[:,currVert[2]]
			pt4=self.mesh.vertexCoords[:,currVert[3]]
			
			sl1,sl2,sl3=pyfrp_integration_module.calcTetSidelengths(pt1,pt2,pt3,pt4)
			
			slsVec.append(sl1)
			slsVec.append(sl2)
			slsVec.append(sl3)
			
		return slsVec
	
	def plotDensity(self,axes=None,hist=True,bins=100,color='b'):
		
		"""Plots the mesh density in x/y/z-direction.
		
		``hist=True`` is recommended, since otherwise plots generally appear fairly 
		noisy. 
		
		.. note:: If no ``axes`` are given or they do not have the necessary size, 
		   will create new ones.
		
		.. image:: ../imgs/pyfrp_mesh/density_plot.png
		
		Keyword Args:
			axes (list): List of ``matplotlib.axes``.
			hist (bool): Summarize densities in bins.
			bins (int): Number of bins used for hist.
			color (str): Color of plot.
			
		Returns:
			list: List of ``matplotlib.axes``.
			
		"""
		
		x,y,z=self.getCellCenters()
		
		volSortedByX,xSorted=pyfrp_misc_module.sortListsWithKey(self.mesh.getCellVolumes(),x)
		volSortedByY,ySorted=pyfrp_misc_module.sortListsWithKey(self.mesh.getCellVolumes(),y)
		volSortedByZ,zSorted=pyfrp_misc_module.sortListsWithKey(self.mesh.getCellVolumes(),z)
		
		if axes==None:
			fig,axes = pyfrp_plot_module.makeSubplot([1,3],titles=["Density(x)","Density(y)","Density(z)"])
		else:
			if len(axes)<3:
				printWarning("axes do not have right have, will create new ones.")
				fig,axes = pyfrp_plot_module.makeSubplot([1,3],titles=["Density(x)","Density(y)","Density(z)"])
		
		if hist:
			xSorted,volSortedByX=pyfrp_misc_module.simpleHist(xSorted,volSortedByX,bins)
			ySorted,volSortedByY=pyfrp_misc_module.simpleHist(ySorted,volSortedByY,bins)
			zSorted,volSortedByZ=pyfrp_misc_module.simpleHist(zSorted,volSortedByZ,bins)
			
		axes[0].plot(xSorted,volSortedByX,color=color)
		axes[1].plot(ySorted,volSortedByY,color=color)
		axes[2].plot(zSorted,volSortedByZ,color=color)
		
		for ax in axes:
			pyfrp_plot_module.redraw(ax)
		
		return axes
	
	def plotCellCenters(self,ax=None,proj=None,color='k',indicateHeight=False,s=5.,roi=None):
		
		"""Plots location of cell centers of mesh.
		
		.. note:: If no ``ax`` are given will create new ones.
		
		If ``proj=[3d]``, will create 3D scatter plot, otherwise project cell centers in 
		2D.
		
		Example:
		
		Create figure
		
		>>> fig,axes = pyfrp_plot_module.makeSubplot([2,2],titles=['2D','2D indicate','3D','3D indicate'],proj=[None,None,'3d','3d'])

		Plot in 4 different ways
		
		>>> mesh.plotCellCenters(ax=axes[0],s=1.)
		>>> mesh.plotCellCenters(ax=axes[1],indicateHeight=True,s=5.)
		>>> mesh.plotCellCenters(ax=axes[2],s=3.)
		>>> mesh.plotCellCenters(ax=axes[3],indicateHeight=True,s=3.)
		
		.. image:: ../imgs/pyfrp_mesh/plotCellCenters.png
		
		Keyword Args:
			ax (matplotlib.axes): Axes to plot in.
			proj (list): List of projections.
			color (str): Color of mesh nodes.
			indicateHeight (bool): Indicate height by color.
			s (float): Size of marker.
			roi (pyfrp.subclasses.pyfrp_ROI): ROI.
			
		Returns:
			matplotlib.axes: Matplotlib axes.
		
		
		"""
		
		if ax==None:
			fig,axes = pyfrp_plot_module.makeSubplot([1,1],titles=["Cell Centers"],proj=proj)
			ax=axes[0]
		
		
		x,y,z = self.getCellCenters()
		
		if roi!=None:
			x=x[roi.meshIdx]
			y=y[roi.meshIdx]
			z=z[roi.meshIdx]
			
		if pyfrp_plot_module.is3DAxes(ax):
			if indicateHeight:
				#color=cm.jet()
				ax.scatter(x,y,z,c=z,s=s)
			else:	
				ax.scatter(x,y,z,c=color,s=s)
		else:
			if indicateHeight:
				ax.scatter(x,y,c=z,s=s)
			else:
				ax.scatter(x,y,c=color,s=s)
		
		pyfrp_plot_module.redraw(ax)
		
		return ax
		
	def printAllAttr(self):
		
		"""Prints out all attributes of mesh object.""" 
		
		print "Mesh of embryo ", self.simulation.embryo.name, " Details."
		printAllObjAttr(self)
	
	def addBoxField(self,volSizeIn,rangeX,rangeY,rangeZ,newFile=True,fnAppendix="_box",comment="newField",run=False,fnOut=None):
		
		"""Adds box field to mesh.
		
		Box fields allow to refine certain areas of the mesh, see also 
		http://gmsh.info/doc/texinfo/gmsh.html#Specifying-mesh-element-sizes .
		
		.. note:: Will keep ``volSizePx`` as volSize outside of the box.
		
		.. note:: If ``fnOut`` is not specified, will do the following: 
		   
		   - If ``newFile=True``, will create new file with path ``fnGeo+/field/custom/fnGeo+fnAppendix.geo``
		   - Else writes into ``fnGeo``.
		
		Args:
			volSizeIn (float): volSize in px inside the box.
			rangeX (list): Range of box field in x-direction given as ``[minVal,maxVal]``.
			rangeY (list): Range of box field in y-direction given as ``[minVal,maxVal]``.
			rangeZ (list): Range of box field in z-direction given as ``[minVal,maxVal]``.
			
		Keyword Args:	
			newFile (bool): Write new mesh into a new .geo file.
			fnAppendix (str): Append this to new file name.
			comment (str): Comment in .geo file before definition of box field.
			run (bool): Run Gmsh on new .geo file afterwards.
			fnOut (str): Path to output geo file.
			dim (int): Dimension of mesh.
			
		Returns:
			str: Path to new .geo file.
			
		"""
		
		if fnOut==None:
			if newFile:
				if "_box" not in self.simulation.embryo.geometry.fnGeo:	
					pyfrp_misc_module.mkdir(self.simulation.embryo.geometry.fnGeo+"/field/")
					pyfrp_misc_module.mkdir(self.simulation.embryo.geometry.fnGeo+"/field/custom/")
					fnOut=os.path.dirname(self.simulation.embryo.geometry.fnGeo)+"/field/custom/"+os.path.basename(self.simulation.embryo.geometry.fnGeo).replace(".geo",fnAppendix+"_"+self.simulation.embryo.geometry.embryo.name+".geo")
				else:
					fnOut=fnOut=self.simulation.embryo.geometry.fnGeo
			else:
				fnOut=self.simulation.embryo.geometry.fnGeo
			
		pyfrp_gmsh_IO_module.addBoxField(self.simulation.embryo.geometry.fnGeo,volSizeIn,self.volSizePx,rangeX,rangeY,rangeZ,comment=comment,fnOut=fnOut)
		self.simulation.embryo.geometry.setFnGeo(fnOut)
		
		if run:
			self.genMesh()
		
		return fnOut
	
	def addBoundaryLayerAroundROI(self,roi,fnOut=None,segments=48,simplify=True,iterations=3,triangIterations=2,
			       fixSurfaces=True,debug=False,volSizePx=None,volSizeLayer=10,thickness=15.,cleanUp=True,
			       approxBySpline=True,angleThresh=0.95,faces='all',onlyAbs=True):
		
		"""Adds boundary layer around ROI to the mesh.
		
		Does this by:
		
			* Generating a stl file describing ROI, see also :py:func:`pyfrp.subclasses.pyfrp_ROI.ROI.render2StlInGeometry`.
			* Read in stl file as new :py:class:`pyfrp.modules.pyfrp_gmsh_geometry.domain` via
			  :py:func:`pyfrp.modules.pyfrp_gmsh_IO_module.readStlFile`.
			* Simplify new geometry via :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.domain.simplifySurfaces`.
			* Extracting selected surfaces via :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.gmshElement.extract`.
			* If selected, surface boundaries are approximated into splines via 
			  :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.gmshElement.extract`.
			* Reading in geometry's .geo file via :py:func:`pyfrp.sublcasses.pyfrp_geometry.geometry.readGeoFile`.
			* Merging ROI geometry into main geometry via :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.domain.merge`.
			* Adding a boundary layer mesh via :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.domain.addBoundaryLayerField`.
			* Adding all surfaces of ROI's domain to boundary layer, see 
			  :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.boundaryLayerField.addFaceListByID`.
			* Writing new .geo file.
			* Setting new .geo file as ``fnGeo``.
			* Running :py:func:`genMesh`.
			* Clean up .stl and .scad files that are not needed anymore.
		
		.. note:: ``volSizeLayer`` only allows a single definition of mesh size in layer. Note that the 
		   :py:class:`pyfrp.modules.pyfrp_gmsh_geometry.boundaryLayerField` class allows different mesh sizes
		   normal and along surfaces. For more information, see its documentation.
		
		.. note:: If no ``fnOut`` is given, will save a new .geo file in same folder as original ``fnGeo`` with subfix:
		   ``fnGeo_roiName_BL.geo``.
		
		.. note:: :py:func:`pyfrp.modules.pyfrp_gmsh_geometry.domain.simplifySurfaces` is not a simple procedure, 
		   we recommend reading its documentation.
		
		If ``volSizePx`` is given, will overwrite mesh's ``volSizePx`` and set it globally at all nodes.
		
		Args:
			roi (pyfrp.subclasses.pyfrp_ROI.ROI): An ROI.
		
		Keyword Args:
			fnOut (str): Path to new .geo file.
			segments (int): Number of segments used for convex hull of surface.
			simplify (bool): Simplify surfaces of stl file.
			iterations (int): Number of iterations used for simplification.
			triangIterations (int): Number of iterations used for subdivision of surfaces.
			addPoints (bool): Allow adding points inside surface triangles.
			fixSurfaces (bool): Allow fixing of surfaces, making sure they are coherent with Gmsh requirements.
			debug (bool): Print debugging messages.
			volSizePx (float): Global mesh density.
			volSizeLayer (float): Boundary layer mesh size.
			thickness (float): Thickness of boundary layer.
			cleanUp (bool): Clean up temporary files when finished.
			approxBySpline (bool): Approximate curvatures by spline.
			angleThresh (float): Threshold angle under which loops are summarized.
			faces (list): List of faces.
			onlyAbs (bool): Take absolute value of faces into account.
			dim (int): Dimension of mesh.
			
		Returns:
			str: Path to new .geo file.
		
		"""
		
		if fnOut==None:
			fnOut=self.simulation.embryo.geometry.fnGeo.replace(".geo","_"+roi.name+"_BL.geo")
		
		#Build stl file for ROI
		fnStl=roi.render2StlInGeometry(segments=segments,fn=fnOut.replace(".geo",".stl"))
		
		#Read in stl file and simplify
		dROI=pyfrp_gmsh_IO_module.readStlFile(fnStl)
		
		if simplify: 
			dROI.simplifySurfaces(iterations=iterations,triangIterations=triangIterations,fixSurfaces=fixSurfaces,debug=debug,addPoints=bool(triangIterations>0))		
	
		# If we extract faces, we have to put them in new ROI
		if faces!='all' and len(faces)>0:
			
			# Grab surfaces
			sfs=dROI.getRuledSurfacesByNormal(faces,onlyAbs=onlyAbs)
			
			# Insert in new domain
			d2=pyfrp_gmsh_geometry.domain()
			for sf in sfs:
				sf.extract(d=d2,debug=debug)
			
			# Overwrite sfs and dROI
			sfs=d2.ruledSurfaces
			dROI=d2
			
		#Approximate complicated curves by splines
		if approxBySpline:
			for sf in sfs:
				sf.lineLoop.approxBySpline(angleThresh=angleThresh)	
		dROI.fixAllLoops()
		
		#Read in geometry and merge 
		dGeo=self.simulation.embryo.geometry.readGeoFile()
		dGeo.merge(dROI)
		
		#Create boundary layer
		blf=dGeo.addBoundaryLayerField(hfar=volSizeLayer,hwall_n=volSizeLayer,hwall_t=volSizeLayer,thickness=thickness,Quads=0.)
		
		#Add surfaces to boundary layer mesh
		blf.addFaceListByID(pyfrp_misc_module.objAttrToList(sfs,'Id'))
		blf.setAsBkgdField()
		
		#Set global volSize
		if volSizePx!=None:
			dGeo.setGlobalVolSize(volSizePx)
			self.volSizePx=volSizePx
		else:
			dGeo.setGlobalVolSize(self.getVolSizePx())
		
		#Write and set as new geometry
		dGeo.writeToFile(fnOut)
		self.simulation.embryo.geometry.setFnGeo(fnOut)
		
		#Generate mesh
		self.genMesh()
		
		#Clean up temporary files
		if cleanUp:
			os.remove(fnStl)
			os.remove(fnStl.replace(".stl",".scad"))
		
		return fnOut
		
	
	def getMaxNodeDistance(self):
		
		"""Returns maximum node distance in x/y/z direction.
		
		Returns:
			tuple: Tuple containing:
			
				* dmaxX (float): Maximum distance in x-direction
				* dmaxY (float): Maximum distance in y-direction
				* dmaxZ (float): Maximum distance in z-direction
				
		"""
		
		distances=self.mesh.cellDistanceVectors
		
		return max(distances[0]),max(distances[1]),max(distances[2])
		
	def getCellCenters(self):
		
		"""Returns cell centers of mesh.
		
		If the mesh is 2-dimensional, places all nodes at ``z=self.simulation.embryo.sliceHeight``.
		
		Returns:
			tuple: Tuple containing:
			
				* x (list): x-coordinates of cells.
				* y (list): y-coordinates of cells.
				* z (list): z-coordinates of cells.
				
		"""
		
		if self.mesh==None:
			return [],[],[]
		
		if len(self.mesh.getCellCenters())==3:
			return self.mesh.getCellCenters()
		else:
			z=self.simulation.embryo.sliceHeightPx*np.ones((len(self.mesh.getCellCenters()[0]),))
			return self.mesh.getCellCenters()[0],self.mesh.getCellCenters()[1],z
		
		


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Optimization module for PyFRAP toolbox.

Currently contains all functions necessary to transform a constrained FRAP optimization problem into
a unconstrained one, making it suitable to Nelder-Mead optimization algorithm. 

"""

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Numpy/Scipy
import numpy as np

#PyFRAP
import pyfrp_fit_module 

from pyfrp_term_module import *

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

def constrObjFunc(x,fit,debug,ax,returnFit):
	
	"""Objective function when using Constrained Nelder-Mead.
	
	Calls :py:func:`pyfrp.modules.pyfrp_optimization_module.xTransform` to transform x into
	constrained version, then uses :py:func:`pyfrp.modules.pyfrp_fit_module.FRAPObjFunc` to 
	find SSD.
	
	Args:
		x (list): Input vector, consiting of [D,(prod),(degr)].
		fit (pyfrp.subclasses.pyfrp_fit): Fit object.
		debug (bool): Display debugging output and plots.
		ax (matplotlib.axes): Axes to display plots in.
		returnFit (bool): Return fit instead of SSD.
	
	Returns:
		 float: SSD of fit. Except ``returnFit==True``, then will return fit itself. 
	"""
	
	
	LBs, UBs = buildBoundLists(fit)
	
	x=xTransform(x,LBs,UBs)

	ssd=pyfrp_fit_module.FRAPObjFunc(x,fit,debug,ax,returnFit)
	
	return ssd

def xTransform(x,LB,UB):
	
	"""Transforms ``x`` into constrained form, obeying upper 
	bounds ``UB`` and lower bounds ``LB``.
	
	.. note:: Will add tiny offset to LB(D), to avoid singularities.
	
	Idea taken from http://www.mathworks.com/matlabcentral/fileexchange/8277-fminsearchbnd--fminsearchcon
	
	Args:
		x (list): Input vector, consiting of [D,(prod),(degr)].
		LB (list): List of lower bounds for ``D,prod,degr``.
		UB (list): List of upper bounds for ``D,prod,degr``.
	
	Returns:
		list: Transformed x-values. 
	"""
	
	#Make sure everything is float
	x=np.asarray(x,dtype=np.float64)
	LB=np.asarray(LB,dtype=np.float64)
	UB=np.asarray(UB,dtype=np.float64)
	
	#Check if LB_D==0, then add a little noise to it so we do not end up with xtrans[D]==0 and later have singularities when scaling tvec
	if LB[0]==0:
		LB[0]=1E-10
	
	#Determine number of parameters to be fitted
	nparams=len(x)

	#Make empty vector
	xtrans = np.zeros(np.shape(x))
	
	# k allows some variables to be fixed, thus dropped from the
	# optimization.
	k=0

	for i in range(nparams):

		#Upper bound only
		if UB[i]!=None and LB[i]==None:
		
			xtrans[i]=UB[i]-x[k]**2
			k=k+1
			
		#Lower bound only	
		elif UB[i]==None and LB[i]!=None:
			
			xtrans[i]=LB[i]+x[k]**2
			k=k+1
		
		#Both bounds
		elif UB[i]!=None and LB[i]!=None:
			
			xtrans[i] = (np.sin(x[k])+1.)/2.*(UB[i] - LB[i]) + LB[i]
			xtrans[i] = max([LB[i],min([UB[i],xtrans[i]])])
			k=k+1
		
		#No bounds
		elif UB[i]==None and LB[i]==None:
		
			xtrans[i] = x[k]
			k=k+1
			
		#Note: The original file has here another case for fixed variable, but since we made the decision earlier which when we call frap_fitting, we don't need this here.
	
	return xtrans	
		
def transformX0(x0,LB,UB):
	
	"""Transforms ``x0`` into constrained form, obeying upper 
	bounds ``UB`` and lower bounds ``LB``.
	
	Idea taken from http://www.mathworks.com/matlabcentral/fileexchange/8277-fminsearchbnd--fminsearchcon
	
	Args:
		x0 (list): Input initial vector, consiting of [D,(prod),(degr)].
		LB (list): List of lower bounds for ``D,prod,degr``.
		UB (list): List of upper bounds for ``D,prod,degr``.
	
	Returns:
		list: Transformed x-values. 
	"""
	
	x0u = list(x0)
	
	nparams=len(x0)
	
	k=0
	for i in range(nparams):
		
		#Upper bound only
		if UB[i]!=None and LB[i]==None:
			if UB[i]<=x0[i]:
				x0u[k]=0
			else:
				x0u[k]=sqrt(UB[i]-x0[i])	
			k=k+1
			
		#Lower bound only
		elif UB[i]==None and LB[i]!=None:
			if LB[i]>=x0[i]:
				x0u[k]=0
			else:
				x0u[k]=np.sqrt(x0[i]-LB[i])	
			k=k+1
		
		
		#Both bounds
		elif UB[i]!=None and LB[i]!=None:
			if UB[i]<=x0[i]:
				x0u[k]=np.pi/2
			elif LB[i]>=x0[i]:
				x0u[k]=-np.pi/2
			else:
				x0u[k] = 2*(x0[i] - LB[i])/(UB[i]-LB[i]) - 1;
				#shift by 2*pi to avoid problems at zero in fminsearch otherwise, the initial simplex is vanishingly small
				x0u[k] = 2*np.pi+np.arcsin(max([-1,min(1,x0u[k])]));
			k=k+1
		
		#No bounds
		elif UB[i]==None and LB[i]==None:
			x0u[k] = x[i]
			k=k+1
	
	return x0u

def buildBoundLists(fit):
	
	"""Builds list of lower bounds and upper bounds.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit): Fit object.
		
	Returns:
		tuple: Tuple containing: 
		
			* LBs (list): List of lower bounds.
			* UBs (list): List of upper bounds.
			
	
	
	"""
	
	LBs=[fit.LBD]+int(fit.fitProd)*[fit.LBProd]+int(fit.fitDegr)*[fit.LBDegr]+len(fit.ROIsFitted)*[fit.LBEqu]
	UBs=[fit.UBD]+int(fit.fitProd)*[fit.UBProd]+int(fit.fitDegr)*[fit.UBDegr]+len(fit.ROIsFitted)*[fit.UBEqu]
	
	return LBs,UBs


import sys
import argparse


# Create parser
parser = argparse.ArgumentParser(description='PyFRAP command line.')

parser.add_argument('-f','--fIn', dest='fIn',action='append',help='Input file names. Can be either embryo files or paths to image data. Note that all inputs should be of the same type.')

# Options for project creation

# Options for image analysis
parser.add_argument('-G','--gaussian', dest='gaussian', action='const_store',help='Perform gaussian blur. Specified as tuple flag,sigma',default="0,5")
parser.add_argument('-M','--median', dest='median', action='const_store',help='Perform median filter. Specified as tuple flag,radius',default="0,2")
parser.add_argument('-F','--flatten', dest='flatten', action='const_store',help='Perform flattening. Specified as tuple flag,pathToFlattenData',default="0,")
parser.add_argument('-B','--bkgd', dest='bkgd', action='const_store',help='Perform background subtraction. Specified as tuple flag,pathToBkgdData',default="0,")
parser.add_argument('-N','--norm', dest='norm', action='const_store',help='Perform normalization. Specified as tuple flag,pathToPreData',default="0,")

# Options for meshing



# Options for simulations
parser.add_argument('--IC', dest='IC', action='const_store',help='Choose initial conditions',default=4)
parser.add_argument('-t','--tsteps', dest='tsteps', action='const_store',help='Number of simulations timesteps',default=3000)
parser.add_argument('-s','--stepping', dest='stepping', action='const_store',help='Time stepping (0=linear,1=log)',default=1)
parser.add_argument('--IC', dest='IC', action='const_store',help='Choose initial conditions',default=4)

# Options for fitting








args = parser.parse_args()


print args

# Choose what kind of data is given and what to do with it.
#inputTypes=[]
#for inp in args.fIn:
	#if os.path.isdir(inp):
		#fns=os.listdir(inp)
		
		
		
		#for f in fns:
			#if os.path.isdir(os.path.join(inp,f)):
				#pass
			#else:
				#path,ext=os.path.splitext(inp)
				
				#if ext in ['.czi','.lsm','.tif','']
				
				
	#else:
		#path,ext=os.path.splitext(inp)
		
		#if ext=='.emb':
			#inputTypes.append('embryo')
		#elif ext=='.mol':
			#inputTypes.append('molecule')
		#else:
			#printWarning("Unknown input type " + inp)
			
			
		
		





#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#PyQT Dialogs for fit class
#(1) 

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#QT
from PyQt4 import QtGui, QtCore

#PyFRAP GUI classes
import pyfrp_gui_basics

#PyFRAP modules
from pyfrp.modules.pyfrp_term_module import *
from pyfrp.modules import pyfrp_misc_module

#Numpy/Scipy
import numpy as np

#Misc 
import os


#===================================================================================================================================
#Dialog for editing pinning settings
#===================================================================================================================================

class defaultPinningDialog(pyfrp_gui_basics.basicSettingsDialog):
	
	def __init__(self,embryo,parent):
		
		super(defaultPinningDialog,self).__init__(parent)
		
		self.embryo = embryo
		
		self.useMin=True
		self.useMax=True
		self.debug=False
		
		#Labels
		self.lblUseMin = QtGui.QLabel("Use Min. Value for Background:", self)
		self.lblUseMax = QtGui.QLabel("Use Max. Value for Norming:", self)
		self.lblDebug = QtGui.QLabel("Show Debugging Output:", self)
		
		#Checkboxes
		self.cbUseMax = QtGui.QCheckBox('', self)
		self.cbUseMin = QtGui.QCheckBox('', self)
		self.cbDebug = QtGui.QCheckBox('', self)
		
		self.updateCBs()
		
		self.connect(self.cbUseMax, QtCore.SIGNAL('stateChanged(int)'), self.checkUseMax)
		self.connect(self.cbUseMin, QtCore.SIGNAL('stateChanged(int)'), self.checkUseMin)
		self.connect(self.cbDebug, QtCore.SIGNAL('stateChanged(int)'), self.checkDebug)
		
		#layout
		self.grid.addWidget(self.lblUseMin,1,1)
		self.grid.addWidget(self.lblUseMax,2,1)
		self.grid.addWidget(self.lblDebug,3,1)
		
		self.grid.addWidget(self.cbUseMin,1,2)
		self.grid.addWidget(self.cbUseMax,2,2)
		self.grid.addWidget(self.cbDebug,3,2)
		
		
		self.setWindowTitle("Basic Pinning Dialog")
		self.show()
	
	def updateCBs(self):	
		self.cbUseMax.setCheckState(2*int(self.useMax))
		self.cbUseMin.setCheckState(2*int(self.useMin))
		self.cbDebug.setCheckState(2*int(self.debug))
		
	def checkDebug(self,val):
		self.debug=bool(2*val)
	
	def checkUseMax(self,val):
		self.useMax=bool(2*val)
		
	def checkUseMin(self,val):
		self.useMin=bool(2*val)
	
	def donePressed(self):
		bkgdVal,normVal=self.embryo.computePinVals(useMin=self.useMin,useMax=self.useMax,bkgdVal=None,debug=self.debug)
		self.embryo.pinAllROIs(bkgdVal=bkgdVal,normVal=normVal,bkgdValSim=bkgdVal,normValSim=normVal,debug=self.debug)
		self.done(1)
		return
		
#===================================================================================================================================
#Dialog for editing ideal pinning settings
#===================================================================================================================================

class idealPinningDialog(defaultPinningDialog):
	
	def __init__(self,embryo,parent):
		
		super(idealPinningDialog,self).__init__(embryo,parent)
		
		self.switchThresh=0.95
		self.bkgdName='Bleached Square'
		self.normName='Slice'
		self.sepSim=True
		
		#Labels
		self.lblBkgd = QtGui.QLabel("ROI for Background Calculation:", self)
		self.lblNorm = QtGui.QLabel("ROI for Norm Calculation:", self)
		
		self.lblBkgdVal = QtGui.QLabel("", self)
		self.lblNormVal = QtGui.QLabel("", self)
		
		self.lblSepSim = QtGui.QLabel("Pin simulation seperately:", self)
		self.lblSwitchThresh = QtGui.QLabel("Switch Threshold:", self)
		
		self.updateBkgdROILbl()
		self.updateNormROILbl()

		#LineEdits
		self.doubleValid=QtGui.QDoubleValidator()
		self.qleSwitchThresh = QtGui.QLineEdit(str(self.switchThresh))
		self.qleSwitchThresh.setValidator(self.doubleValid)
		self.qleSwitchThresh.editingFinished.connect(self.setSwitchThresh)
		
		#Buttons
		self.btnSetBkgdROI=QtGui.QPushButton('Change')
		self.btnSetBkgdROI.connect(self.btnSetBkgdROI, QtCore.SIGNAL('clicked()'), self.setBkgdROI)
		
		self.btnSetNormROI=QtGui.QPushButton('Change')
		self.btnSetNormROI.connect(self.btnSetNormROI, QtCore.SIGNAL('clicked()'), self.setNormROI)
		
		
		#Checkboxes
		self.cbSepSim = QtGui.QCheckBox('', self)
		self.cbSepSim.setCheckState(2*int(self.sepSim))
	
		self.connect(self.cbSepSim, QtCore.SIGNAL('stateChanged(int)'), self.checkSepSim)
		
		#Layout
		nRows=self.grid.rowCount()
		self.grid.addWidget(self.lblBkgd,nRows+1,1)
		self.grid.addWidget(self.lblNorm,nRows+2,1)
		self.grid.addWidget(self.lblSwitchThresh,nRows+3,1)
		self.grid.addWidget(self.lblSepSim,nRows+4,1)
		
		self.grid.addWidget(self.lblBkgdVal,nRows+1,2)
		self.grid.addWidget(self.lblNormVal,nRows+2,2)
		self.grid.addWidget(self.qleSwitchThresh,nRows+3,2)
		self.grid.addWidget(self.cbSepSim,nRows+4,2)
		
		self.grid.addWidget(self.btnSetBkgdROI,nRows+1,3)
		self.grid.addWidget(self.btnSetNormROI,nRows+2,3)
		
		self.setWindowTitle("Ideal Pinning Dialog")
		self.show()
		
	def setSwitchThresh(self):
		self.switchThresh=float(str(self.qleSwitchThresh.text()))
	
	def checkSepSim(self,val):
		self.sepSim=bool(2*val)	
		
	def setBkgdROI(self):
		
		nameList=pyfrp_misc_module.objAttrToList(self.embryo.ROIs,'name')
		
		selectorDialog = pyfrp_gui_basics.basicSelectorDialog(nameList,self)
		if selectorDialog.exec_():
			selectedROIName = selectorDialog.getItem()
			if selectedROIName==None:
				return
		
		self.bkgdName=selectedROIName
		self.updateBkgdROILbl()	
			
	def updateBkgdROILbl(self):
		nameList=pyfrp_misc_module.objAttrToList(self.embryo.ROIs,'name')
		if self.bkgdName in nameList:
			pass
		else:
			self.bkgdName=""
			
		self.lblBkgdVal.setText(self.bkgdName)
	
	def setNormROI(self):
		
		nameList=pyfrp_misc_module.objAttrToList(self.embryo.ROIs,'name')
		
		selectorDialog = pyfrp_gui_basics.basicSelectorDialog(nameList,self)
		if selectorDialog.exec_():
			selectedROIName = selectorDialog.getItem()
			if selectedROIName==None:
				return
		
		self.normName=selectedROIName
		self.updateNormROILbl()	
			
	def updateNormROILbl(self):
		nameList=pyfrp_misc_module.objAttrToList(self.embryo.ROIs,'name')
		if self.normName in nameList:
			pass
		else:
			self.normName=""
			
		self.lblNormVal.setText(self.normName)
		
	def donePressed(self):

		bkgdVal, normVal, bkgdValSim, normValSim=self.embryo.computeIdealFRAPPinVals(bkgdName=self.bkgdName,normName=self.normName,debug=self.debug,useMin=self.useMin,useMax=self.useMax,sepSim=self.sepSim,switchThresh=self.switchThresh)
		self.embryo.pinAllROIs(bkgdVal=bkgdVal,normVal=normVal,bkgdValSim=bkgdValSim,normValSim=normValSim,debug=self.debug)
		self.done(1)
		return	
		
		


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PyFRAP: A Python based FRAP analysis tool box. Gui package.
"""

#PyFRAP main GUI
from . import pyfrp_gui_builder
from . import pyfrp_gui_vtk
from . import pyfrp_app

#from . import pyfrp_term

